################## needed addition ######################
"""
Description: This file already contains a desal model, but we need an estimate of the desal unit size, particularly mass and footprint (m^2)
Sources:
    - [1] Singlitico 2021 (use this as a jumping off point, I think there may be other good sources available)
    - [2] See sources in existing model below and the model itself
Args:
    - electrolyzer_rating (float): electrolyzer rating in MW
    - input and output values from RO_desal() below
    - others may be added as needed
Returns (can be from separate functions and/or methods as it makes sense):
    - mass (float): approximate mass of the desalination system (kg or tonnes)
    - footprint (float): approximate area required for the desalination system (m^2)
"""


#################### existing model ########################

## High-Pressure Reverse Osmosis Desalination Model
"""
Python model of High-Pressure Reverse Osmosis Desalination (HPRO).

Reverse Osmosis (RO) is a membrane separation process. No heating or phase change is necessary.
The majority of energy required is for pressurizing the feed water.

A typical RO system is made up of the following basic components:
Pre-treatment: Removes suspended solids and microorganisms through sterilization, fine filtration and adding chemicals to inhibit precipitation.
High-pressure pump: Supplies the pressure needed to enable the water to pass through the membrane (pressure ranges from 54 to 80 bar for seawater).
Membrane Modules: Membrane assembly consists of a pressure vessel and the membrane. Either sprial wound membranes or hollow fiber membranes are used.
Post-treatment: Consists of sterilization, stabilization, mineral enrichment and pH adjustment of product water.
Energy recovery system: A system where a portion of the pressure energy of the brine is recovered.

Costs are in 2013 dollars
"""
import sys
import numpy as np
from greenheart.to_organize.H2_Analysis.simple_cash_annuals import simple_cash_annuals



def RO_desal_eco(freshwater_kg_per_hr, salinity):
    """
    param: freshwater_kg_per_hr: Maximum freshwater requirements of system [kg/hr]

    param: salinity: (str) "Seawater" >18,000 ppm or "Brackish" <18,000 ppm
        
    output: feedwater_m3_per_hr: feedwater flow rate [m^3/hr] 
    
    output: desal_power: reqired power [kW] 

    output: desal_capex: Capital cost [USD]

    output: desal_opex: OPEX (USD/yr)

    Costs from: https://pdf.sciencedirectassets.com/271370/1-s2.0-S0011916408X00074/1-s2.0-S0011916408002683/main.pdf?X-Amz-Security-Token=IQoJb3JpZ2luX2VjEEcaCXVzLWVhc3QtMSJGMEQCIBNfL%2Frp%2BWpMGUW7rWBm3dkXztvOFIdswOdqI23VkBTGAiALG4NJuAiUkzKnukw233sXHF1OFBPnogJP1ZkboPkaiSrVBAjA%2F%2F%2F%2F%2F%2F%2F%2F%2F%2F8BEAUaDDA1OTAwMzU0Njg2NSIMWZ%2Fh3cDnrPjUJMleKqkELlVPKjinHYk85KwguMS3panLr1RRD9qkoxIASocYCbkvKLE9xW%2BT8QMCtEaH3Is7NRZ2Efc6YFQiO0DHbRzXYTfgz6Er5qqvSAFTrfgp%2B5bB3NYvtDI3kEGH%2F%2BOrEiL8iDK9TmgUjojvnKt86zidswBSDWrzclxcLrw6dfsqZf6dVjJT2g3Cyy8LKnP9vc33tCbACRLeszW1Zce%2BTlBbON22W%2FJq0qLcXDxI9JpRDqL8T%2Fo7SsetEif2DWovTLnv%2B%2FX2tJotFp630ZTVpd37ukGtanjAr5pl0nHgjnUtOJVtNksHQwc8XElFpBGKEXmvRo2uZJFd%2BeNtPEB1dWIZlZul6B8%2BJ7D%2FSPJsclPfpkMU92YUStQpw4Mc%2FOJFCILFyb4416DsL6PVWsdcYu9bbry8c0hQGZlE7oXTFoUy9SKdpEOguXAUi3X4JxjZisy3esVH8zNS3%2FiFsNr2FkTB6MLaSjSKj344AuDCkQYZ7CnenAiCHgf4a2tSnfiXzAvAFnpeQkr4iCnZOQ4Eis6L3fVRpWlluX5HUpbvUMN6rvtmAzq0APJn1b3NmFHy4ORoemTGvmI%2FHTRYKuAu257XBMe7X1qAJlnmpt6yGXrelXCz%2FmUvmbT1SzxETA5ss4KR0OM4YdXNnFLUrsV44ZkUM%2B8FlwZr%2F%2FePjz4QeG4ApR821IYTyre3%2FY%2BBZxaMs5AcXKiTHGwfE7CDi%2BQQ7CnDKk0lleZcas6kxzDl9%2BmeBjqqAeZhBVwd5sEx6aDGxAQC0eWpux6HauoVfuPOCkkv621szF0kTBqcoOlJlJav4eUPW4efAzBremirjiRLI2GdP72lVqXz9oaCg5NFXeKJAWbWkLdzHnDOu8ecSUPn%2F0jcR2IO2mznLspx6wKQA%2BAPEVGgptkwZtDqHcw8FNx7Q8tWJ1C4qL1bEMl0%2FatDXOHiJfuzCFp4%2B4uijTNfpVXO%2BzYQuNJA7ZNUMroa&X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Date=20230201T155950Z&X-Amz-SignedHeaders=host&X-Amz-Expires=300&X-Amz-Credential=ASIAQ3PHCVTY7RLVF2MG%2F20230201%2Fus-east-1%2Fs3%2Faws4_request&X-Amz-Signature=a3770ee910f7f78c94bb84206538810ca03f7a653183191b3794c633b9e3f08f&hash=2e8904ff0d2a6ef567a5894d5bb773524bf1a90bc3ed88d8592e3f9d4cc3c531&host=68042c943591013ac2b2430a89b270f6af2c76d8dfd086a07176afe7c76c2c61&pii=S0011916408002683&tid=spdf-27339dc5-0d03-4078-a244-c049a9bb014d&sid=50eb5802654ba84dc80a5675e9bbf644ed4dgxrqa&type=client&tsoh=d3d3LnNjaWVuY2VkaXJlY3QuY29t&ua=0f1650585c05065559515c&rr=792be5868a1a8698&cc=us
    
    A desal system capacity is given as desired freshwater flow rate [m^3/hr]
    """

    freshwater_density = 997    #[kg/m^3]
    freshwater_m3_per_hr = freshwater_kg_per_hr / freshwater_density 
    desal_capacity = freshwater_m3_per_hr

    if salinity == "Seawater":
        # SWRO: Sea Water Reverse Osmosis, water >18,000 ppm 
        # Water recovery
        recovery_ratio = 0.5    #https://www.usbr.gov/research/dwpr/reportpdfs/report072.pdf
        feedwater_m3_per_hr = freshwater_m3_per_hr / recovery_ratio

        # Power required
        energy_conversion_factor = 4.0  #[kWh/m^3] SWRO energy_conversion_factor range 2.5 to 4.0 kWh/m^3
                                        #https://www.sciencedirect.com/science/article/pii/S0011916417321057
        desal_power = freshwater_m3_per_hr * energy_conversion_factor


    elif salinity == "Brackish":
        # BWRO: Brakish water Reverse Osmosis, water < 18,000 ppm   
        # Water recovery
        recovery_ratio = 0.75    #https://www.usbr.gov/research/dwpr/reportpdfs/report072.pdf
        feedwater_m3_per_hr = freshwater_m3_per_hr / recovery_ratio

        # Power required
        energy_conversion_factor = 1.5  #[kWh/m^3] BWRO energy_conversion_factor range 1.0 to 1.5 kWh/m^3
                                        #https://www.sciencedirect.com/science/article/pii/S0011916417321057    
                                        
        desal_power = freshwater_m3_per_hr * energy_conversion_factor
    
    else:
        raise Exception("Salinity parameter must be set to Brackish or Seawater")

    # Costing
    # https://www.nrel.gov/docs/fy16osti/66073.pdf
    desal_capex = 32894 * (freshwater_density * desal_capacity / 3600) # [USD]

    desal_opex = 4841 * (freshwater_density * desal_capacity / 3600) # [USD/yr]

    '''Mass and Footprint
    Based on Commercial Industrial RO Systems
    https://www.appliedmembranes.com/s-series-seawater-reverse-osmosis-systems-2000-to-100000-gpd.html
    
    All Mass and Footprint Estimates are estimated from Largest RO System:
    S-308F
    -436 m^3/day
    -6330 kg
    -762 cm (L) x 112 cm (D) x 183 cm (H)

    436 m^3/day = 18.17 m^3/hr = 8.5 m^2, 6330 kg
    1 m^3/hr = .467 m^2, 346.7 kg

    Voltage Codes
    460 or 480v/ 3ph/ 60 Hz
    '''
    desal_mass_kg = freshwater_m3_per_hr * 346.7    #[kg]
    desal_size_m2 = freshwater_m3_per_hr * .467     #[m^2]

    
    return desal_capacity, feedwater_m3_per_hr, desal_power, desal_capex, desal_opex, desal_mass_kg, desal_size_m2

if __name__ == '__main__':
    desal_freshwater_kg_hr = 75000
    salinity = 'Brackish'
    test = RO_desal_eco(desal_freshwater_kg_hr,salinity)
    print(test)


# general imports
from pathlib import Path

# GreenHEART imports
from greenheart.simulation.greenheart_simulation import GreenHeartSimulationConfig
from greenheart.tools.optimization.gc_run_greenheart import run_greenheart


DATA_PATH = Path(__file__).parent / "input"


# run the stuff
if __name__ == "__main__":
    # load inputs as needed
    turbine_model = "lbw_6MW"
    filename_turbine_config = DATA_PATH / f"turbines/{turbine_model}.yaml"
    filename_floris_config = DATA_PATH / "floris/floris_input_lbw_6MW.yaml"
    filename_hopp_config = DATA_PATH / "plant/hopp_config_tx.yaml"
    filename_greenheart_config = DATA_PATH / "plant/greenheart_config_onshore_tx.yaml"

    config = GreenHeartSimulationConfig(
        filename_hopp_config,
        filename_greenheart_config,
        filename_turbine_config,
        filename_floris_config,
        verbose=True,
        show_plots=False,
        save_plots=True,
        use_profast=True,
        post_processing=True,
        incentive_option=1,
        plant_design_scenario=9,
        output_level=7,
    )

    # for analysis
    prob, config = run_greenheart(config, run_only=True)

    # for optimization
    # prob, config = run_greenheart(config, run_only=False)
    
    lcoe = prob.get_val("lcoe", units="USD/(MW*h)")
    lcoh = prob.get_val("lcoh", units="USD/kg")
    lcoa = prob.get_val("lcoa", units="USD/kg")

    print("LCOE: ", lcoe, "[$/MWh]")
    print("LCOH: ", lcoh, "[$/kg]")
    print("LCOA: ", lcoa, "[$/kg]")



"""
Author: Kaitlin Brunik
Created: 7/20/2023
Institution: National Renewable Energy Lab
Description: This file outputs capital and operational costs of salt cavern hydrogen storage.
It needs to be updated to with operational dynamics.
Costs are in 2018 USD

Sources:
    - [1] Papadias 2021: https://www.sciencedirect.com/science/article/pii/S0360319921030834?via%3Dihub
    - [2] Papadias 2021: Bulk Hydrogen as Function of Capacity.docx documentation at hopp/hydrogen/h2_storage
    - [3] HDSAM V4.0 Gaseous H2 Geologic Storage sheet
"""

import numpy as np
from greenheart.simulation.technologies.hydrogen.h2_transport.h2_compression import Compressor

class SaltCavernStorage():
    """
    - Costs are in 2018 USD
    """

    def __init__(self, input_dict):  
        """
        Initialize SaltCavernStorage.

        Args:
            input_dict (dict):
                - h2_storage_kg (float): total capacity of hydrogen storage [kg]
                - storage_duration_hrs (float): (optional if h2_storage_kg set) [hrs]
                - flow_rate_kg_hr (float): (optional if h2_storage_kg set) [kg/hr]
                - system_flow_rate (float): [kg/day]
                - model (str): ('papadias' or 'hdsam')
                - labor_rate (float): (optional, default: 37.40) [$2018/hr]
                - insurance (float): (optional, default: 1%) [decimal percent]
                - property_taxes (float): (optional, default: 1%) [decimal percent]
                - licensing_permits (float): (optional, default: 0.01%) [decimal percent]
        Returns:
            - salt_cavern_storage_capex_per_kg (float): the installed capital cost per kg h2 in 2018 [USD/kg]
            - installed_capex (float): the installed capital cost in 2018 [USD] (including compressor)
            - storage_compressor_capex (float): the installed capital cost in 2018 for the compressor [USD]
            - total_opex (float): the OPEX (annual, fixed) in 2018 excluding electricity costs [USD/kg-yr]
            - output_dict (dict):
                - salt_cavern_storage_capex (float): installed capital cost in 2018 [USD]
                - salt_cavern_storage_opex (float): OPEX (annual, fixed) in 2018  [USD/yr]
        """         
        self.input_dict = input_dict
        self.output_dict = {}

        #inputs
        if 'h2_storage_kg' in input_dict:
            self.h2_storage_kg = input_dict['h2_storage_kg']        #[kg]
        elif 'storage_duration_hrs' and 'flow_rate_kg_hr' in input_dict:
            self.h2_storage_kg = input_dict['storage_duration_hrs'] * input_dict['flow_rate_kg_hr']  
        else:
            raise Exception('input_dict must contain h2_storage_kg or storage_duration_hrs and flow_rate_kg_hr')

        if 'system_flow_rate' not in input_dict.keys():
                raise ValueError("system_flow_rate required for salt cavern storage model.")
        else:
            self.system_flow_rate = input_dict['system_flow_rate']

        if 'model' in input_dict:
            self.model = input_dict['model']    #[papadias, hdsam]
        else:
            raise Exception('input_dict must contain model type of either `papadias` or `hdsam`')

        self.labor_rate = input_dict.get('labor_rate', 37.39817) # $(2018)/hr
        self.insurance = input_dict.get('insurance', 1/100) # % of total capital investment
        self.property_taxes = input_dict.get('property_taxes', 1/100) # % of total capital investment
        self.licensing_permits = input_dict.get('licensing_permits',0.1/100) # % of total capital investment
        self.comp_om = input_dict.get('compressor_om',4/100)    # % of compressor capital investment
        self.facility_om = input_dict.get('facility_om', 1/100) # % of facility capital investment minus compressor capital investment

    def salt_cavern_capex(self):
        """
        Calculates the installed capital cost of salt cavern hydrogen storage
        Returns:
            - salt_cavern_capex_per_kg (float): the installed capital cost per kg h2 in 2018 [USD/kg]
            - installed_capex (float): the installed capital cost in 2018 [USD] (including compressor)
            - storage_compressor_capex (float): the installed capital cost in 2018 for the compressor [USD]
            - output_dict (dict):
                - salt_cavern_capex (float): installed capital cost in 2018 [USD]
        """

        if self.model == 'papadias':
            # Installed capital cost
            a = 0.092548
            b = 1.6432
            c = 10.161
            self.salt_cavern_storage_capex_per_kg = np.exp(a*(np.log(self.h2_storage_kg/1000))**2 - b*np.log(self.h2_storage_kg/1000) + c)  # 2019 [USD] from Papadias [2]
            self.installed_capex = self.salt_cavern_storage_capex_per_kg * self.h2_storage_kg
            cepci_overall = 1.29/1.30 # Convert from $2019 to $2018
            self.installed_capex = cepci_overall * self.installed_capex
            self.output_dict['salt_cavern_storage_capex'] = self.installed_capex

            outlet_pressure = 120 # Max outlet pressure of salt cavern in [1]
            n_compressors = 2
            storage_compressor = Compressor(outlet_pressure,self.system_flow_rate,n_compressors=n_compressors)
            storage_compressor.compressor_power()
            motor_rating, power = storage_compressor.compressor_system_power()
            if motor_rating > 1600:
                n_compressors += 1
                storage_compressor = Compressor(outlet_pressure,self.system_flow_rate,n_compressors=n_compressors)
                storage_compressor.compressor_power()
                motor_rating, power = storage_compressor.compressor_system_power()
            comp_capex,comp_OM = storage_compressor.compressor_costs()
            cepci = 1.36/1.29 # convert from $2016 to $2018
            self.comp_capex = comp_capex*cepci
        elif self.model == 'hdsam':
            raise NotImplementedError
        return self.salt_cavern_storage_capex_per_kg, self.installed_capex, self.comp_capex

    def salt_cavern_opex(self):
        """
        Calculates the operation and maintenance costs excluding electricity costs for the salt cavern hydrogen storage
        - Returns:
            - total_opex (float): the OPEX (annual, fixed) in 2018 excluding electricity costs [USD/kg-yr]
            - output_dict (dict):
                - salt_cavern_storage_opex (float): OPEX (annual, fixed) in 2018  [USD/yr]
        """
        # Operations and Maintenace costs [3]
        # Labor 
        # Base case is 1 operator, 24 hours a day, 7 days a week for a 100,000 kg/day average capacity facility.  Scaling factor of 0.25 is used for other sized facilities
        annual_hours = 8760 * (self.system_flow_rate/100000)**0.25
        self.overhead = 0.5 
        labor = (annual_hours*self.labor_rate) * (1+self.overhead) # Burdened labor cost
        insurance = self.insurance * self.installed_capex
        property_taxes = self.property_taxes * self.installed_capex
        licensing_permits = self.licensing_permits * self.installed_capex
        comp_op_maint = self.comp_om * self.comp_capex 
        facility_op_maint = self.facility_om * (self.installed_capex - self.comp_capex)

        # O&M excludes electricity requirements
        total_om = labor+insurance+licensing_permits+property_taxes+comp_op_maint+facility_op_maint
        self.output_dict['salt_cavern_storage_opex'] = total_om
        return total_om


from greenheart.simulation.technologies.hydrogen.h2_transport.h2_export_pipe import run_pipe_analysis
from pytest import approx

# test that we the results we got when the code was recieved
class TestExportPipeline():
    L = 8                   # Length [km]
    m_dot = 1.5            # Mass flow rate [kg/s] assuming 300 MW -> 1.5 kg/s
    p_inlet = 30            # Inlet pressure [bar]
    p_outlet = 10           # Outlet pressure [bar]
    depth = 80              # depth of pipe [m]
    costs = run_pipe_analysis(L,m_dot,p_inlet,p_outlet,depth)
        
    def test_grade(self):
        assert self.costs["Grade"][0] == "X42"
    
    def test_od(self):
        assert self.costs["Outer diameter (mm)"][0] == 168.28
    
    def test_od(self):
        assert self.costs["Inner diameter (mm)"][0] == 162.74

    def test_schedule(self):
        assert self.costs["Schedule"][0] == "S 5S"

    def test_thickness(self):
        assert self.costs["Thickness (mm)"][0] == 2.77
    
    def test_volume(self):
        assert self.costs["volume [m3]"][0] == 12.213769866246679

    def test_weight(self):
        assert self.costs["weight [kg]"][0] == 95755.95575137396

    def test_material_cost(self):
        assert self.costs["mat cost [$]"][0] == 210663.1026530227
    
    def test_labor_cost(self):
        assert self.costs["labor cost [$]"][0] == 1199293.4563603932

    def test_misc_cost(self):
        assert self.costs["misc cost [$]"][0] == 429838.3943856504
    
    def test_row_cost(self): #ROW = right of way
        assert self.costs["ROW cost [$]"][0] == 365317.5476681454

    def test_total_cost_output(self):
        assert self.costs["total capital cost [$]"][0] == 2205112.501067212

    def test_total_capital_cost_sum(self):
        total_capital_cost = self.costs["mat cost [$]"][0] \
                    + self.costs["labor cost [$]"][0] \
                    + self.costs["misc cost [$]"][0] \
                    + self.costs["ROW cost [$]"][0]

        assert self.costs["total capital cost [$]"][0] == total_capital_cost

    def test_annual_opex(self):
        assert self.costs["annual operating cost [$]"][0] == 0.0117*self.costs["total capital cost [$]"][0]

class TestExportPipelineRegion():
    L = 8                   # Length [km]
    m_dot = 1.5            # Mass flow rate [kg/s] assuming 300 MW -> 1.5 kg/s
    p_inlet = 30            # Inlet pressure [bar]
    p_outlet = 10           # Outlet pressure [bar]
    depth = 80              # depth of pipe [m]
    region = 'GP'           # great plains region
    costs = run_pipe_analysis(L,m_dot,p_inlet,p_outlet,depth,region=region)

    def test_material_cost(self):
        assert self.costs["mat cost [$]"][0] == 210663.1026530227
    
    def test_labor_cost(self):
        assert self.costs["labor cost [$]"][0] == 408438.062500015

    def test_misc_cost(self):
        assert self.costs["misc cost [$]"][0] == 184458.92890187018
    
    def test_row_cost(self): #ROW = right of way
        assert self.costs["ROW cost [$]"][0] == 52426.57591258784

    def test_total_cost_output(self):
        assert self.costs["total capital cost [$]"][0] == 855986.6699674957

class TestExportPipelineOverrides():
    L = 8                   # Length [km]
    m_dot = 1.5            # Mass flow rate [kg/s] assuming 300 MW -> 1.5 kg/s
    p_inlet = 30            # Inlet pressure [bar]
    p_outlet = 10           # Outlet pressure [bar]
    depth = 80              # depth of pipe [m]
    labor_in_mi = 1000
    misc_in_mi = 2000
    row_in_mi = 3000
    mat_in_mi = 4000
    costs = run_pipe_analysis(
        L,
        m_dot,
        p_inlet,
        p_outlet,
        depth,
        labor_in_mi=labor_in_mi,
        misc_in_mi=misc_in_mi,
        row_in_mi=row_in_mi,
        mat_in_mi=mat_in_mi,
    )

    def test_material_cost(self):
        assert self.costs["mat cost [$]"][0] == 124469.9746153248

    def test_labor_cost(self):
        assert self.costs["labor cost [$]"][0] == 31117.4936538312

    def test_misc_cost(self):
        assert self.costs["misc cost [$]"][0] == 62234.9873076624

    def test_row_cost(self): #ROW = right of way
        assert self.costs["ROW cost [$]"][0] == 93352.4809614936

    def test_total_cost_output(self):
        assert self.costs["total capital cost [$]"][0] == 311174.936538312


if __name__ == "__main__":
    test_set = TestExportPipeline()



from greenheart.to_organize.plot_power_to_load import plot_power_to_load
from greenheart.to_organize.plot_shortfall_curtailment import plot_shortfall_curtailment
from greenheart.to_organize.plot_battery import plot_battery
import numpy as np
import matplotlib.pyplot as plt

def plot_reopt_results(REoptResultsDF, site_name, atb_year, critical_load_factor,
                               useful_life, tower_height,
                               wind_size_mw, solar_size_mw, storage_size_mw, storage_size_mwh, lcoe,
                               feedstock_cost_h2_via_net_cap_cost_lifetime_h2_hopp, feedstock_cost_h2_levelized_hopp,
                               hybrid_installed_cost, h2a_costs,
                               total_unit_cost_of_hydrogen,
                               output_dir,
                               monthly_separation=False, reopt_was_run=False):
    """
    Plots REopt results (wind, solar, battery power) at 24hr averaged periods over the course of a year
    @param REoptResultsDataframe: Dataframe of REOpt results
    @param monthly_separation: (Boolean) determines whether plots are grouped by months or not (alternative is by hour)
    @param reopt_was_run: (Boolean) indicates whether reopt was run or whether dummy data was loaded
    """
    REoptResultsDF.index = REoptResultsDF.Date

    # Determine either monthly or hourly means and 95% CIs
    if monthly_separation:
        # Group by months
        df_mean = REoptResultsDF.groupby(by=[REoptResultsDF.index.month, REoptResultsDF.index.hour]).mean()
        df_std = REoptResultsDF.groupby(by=[REoptResultsDF.index.month, REoptResultsDF.index.hour]).std()
        df_n = REoptResultsDF.groupby(by=[REoptResultsDF.index.month, REoptResultsDF.index.hour]).count()
        z = 1.96
        df_ci = z * df_std / df_n.applymap(np.sqrt)
        prop_cycle = plt.rcParams['axes.prop_cycle']
        colors = prop_cycle.by_key()['color']
        month_map = {
            1: "Jan", 2: "Feb", 3: "Mar", 4: "Apr", 5: "May", 6: "Jun",
            7: "Jul", 8: "Aug", 9: "Sep", 10: "Oct", 11: "Nov", 12: "Dec"
        }

        mapped = [f"{month_map[m]}-{h}" for m, h in df_mean.index.values]
        y = range(df_mean.index.values.shape[0])

        xticks_major = [x * 24 for x in range(1, 13)]
        xticks_minor = list(range(0, 24 * 12, 6))
        xlabels_major = [month_map[m / 24].ljust(13) for m in xticks_major]
        xlabels_minor = ["", "06", "12", "18"] + ["06", "12", "18", "24"] * 11
    else:
        # Group by hours
        df_mean = REoptResultsDF.groupby(by=[REoptResultsDF.index.hour]).mean()
        df_std = REoptResultsDF.groupby(by=[REoptResultsDF.index.hour]).std()
        df_n = REoptResultsDF.groupby(by=[REoptResultsDF.index.hour]).count()
        z = 1.96
        df_ci = z * df_std / df_n.applymap(np.sqrt)
        prop_cycle = plt.rcParams['axes.prop_cycle']
        colors = prop_cycle.by_key()['color']

        y = range(df_mean.index.values.shape[0])

        xticks_major = [x for x in range(1, 24)]
        xticks_minor = list(range(0, 24))
        xlabels_major = [hr for hr in xticks_major]
        xlabels_minor = xticks_minor

    # Plot 1 - Energy supplied to load
    if not reopt_was_run:
        reopt_not_run_warning = 'WARNING: REOPT WAS NOT RUN. DUMMY DATA LOADED'
    else:
        reopt_not_run_warning = ''

    titletext = '{} \n PV and Wind Power at {} plant | ATB Year {} \n Critical Load Factor (0-1): {:,.2f} \n' \
                ' Wind Size (MW): {:,.2f} | Solar Size (MW): {:,.2f} \n Storage Size (MW): {:,.2f} | Storage Size MWh: {:,.2f} \n' \
                ' HOPP LCOE: {:,.2f}c | H2 Levelized Electrical Cost ($/kg): ${:,.2f} | Hybrid Installed Cost: ${:,.2f} \n' \
                ' H2A Levelized Plant Costs ($/kg): ${:,.2f} | Total Levelized H2 Cost ($/kg): ${:,.2f} \n' \
                ' Total Energy Provided (MWh): {:,.2f} Total Energy Shortfall: {:,.2f} Total Energy Curtailed (MWh) {:,.2f}' \
        .format(reopt_not_run_warning, site_name, atb_year, critical_load_factor, wind_size_mw, solar_size_mw, storage_size_mw, storage_size_mwh,
                lcoe, feedstock_cost_h2_levelized_hopp, hybrid_installed_cost, h2a_costs, total_unit_cost_of_hydrogen,
                np.sum(REoptResultsDF['combined_pv_wind_storage_power_production']),
                np.sum(REoptResultsDF['energy_shortfall']), np.sum(REoptResultsDF['combined_pv_wind_curtailment']))
    filename = "wind_pv_{}_production_atb{}_uselife{}_critlo{}_hh{}.png".format(site_name,
                                                                                     atb_year, useful_life,
                                                                                     critical_load_factor, tower_height)
    save_location = output_dir + filename
    ylim = [0, 10000]
    plot_power_to_load(titletext, df_mean, df_ci, y, ylim,
                       colors, xticks_major, xticks_minor, xlabels_major, xlabels_minor,
                       save_location=save_location)

    # Plot 2 - SHORTFALL & CURTAILMENT
    filename = "wind_pv_{}_shortfall_curtailment_atb{}_uselife{}_critlo{}_hh{}.png".format(site_name,
                                                                                                atb_year, useful_life,
                                                                                                critical_load_factor,
                                                                                                tower_height)
    save_location = output_dir + filename
    ylim = [0, 10000]
    plot_shortfall_curtailment(titletext, df_mean, df_ci, y, ylim,
                               colors, xticks_major, xticks_minor, xlabels_major, xlabels_minor,
                               save_location=save_location)

    # Plot 3 - Battery Only
    filename = "battery_{}_atb{}_uselife{}_critlo{}_hh{}.png".format(site_name,
                                                                          atb_year,
                                                                          useful_life,
                                                                          critical_load_factor,
                                                                          tower_height)
    save_location = output_dir + filename
    ylim = [0, 10000]
    plot_battery(titletext, df_mean, df_ci, y, ylim,
                 colors, xticks_major, xticks_minor, xlabels_major, xlabels_minor,
                 save_location=save_location)


# -*- coding: utf-8 -*-
"""
Created on Wed Oct 19 12:13:58 2022

@author: ereznic2
"""

import ProFAST
import pandas as pd

# Specify file path to PyFAST
import sys
#sys.path.insert(1,'../PyFAST/')



# sys.path.append('../PyFAST/')

# import src.PyFAST as PyFAST

#mat_n_heat_integration = 1

def run_profast_for_steel(plant_capacity_mtpy,plant_capacity_factor,\
    plant_life,levelized_cost_of_hydrogen,electricity_cost, grid_prices_interpolated_USDperMWh,natural_gas_cost,\
        lime_unitcost,
        carbon_unitcost,
        iron_ore_pellet_unitcost,
        o2_heat_integration):

# # Steel plant capacity in metric tonnes per year (eventually import to function)
    # plant_capacity_mtpy = 1162077
    # plant_capacity_factor = 0.9
    # levelized_cost_of_hydrogen = 10
  
    # # Should connect these to something (AEO, Cambium, etc.)
    # natural_gas_cost = 4                        # $/MMBTU
    # electricity_cost = 48.92                    # $/MWh
    # plant_life = 30
    # lime_unitcost = 155.34
    # carbon_unitcost = 218.74
    # iron_ore_pellet_unitcost = 230.52

    model_year_CEPCI = 596.2
    equation_year_CEPCI = 708.8
    
    steel_production_mtpy = plant_capacity_mtpy*plant_capacity_factor
    
    # # Hydrogen cost
    # levelized_cost_of_hydrogen = 7              # $/kg
    # natural_gas_cost = 4                        # $/MMBTU
    # electricity_cost = 48.92                    # $/MWh

    #--------------------- Capital costs and Total Plant Cost ---------------------

    capex_eaf_casting = model_year_CEPCI/equation_year_CEPCI*352191.5237*plant_capacity_mtpy**0.456
    capex_shaft_furnace = model_year_CEPCI/equation_year_CEPCI*489.68061*plant_capacity_mtpy**0.88741
    capex_oxygen_supply = model_year_CEPCI/equation_year_CEPCI*1715.21508*plant_capacity_mtpy**0.64574
    if o2_heat_integration == 1:
       capex_h2_preheating = model_year_CEPCI/equation_year_CEPCI*(1 - 0.4) * (45.69123*plant_capacity_mtpy**0.86564) # Optimistic ballpark estimate of 60% reduction in preheating
       capex_cooling_tower = model_year_CEPCI/equation_year_CEPCI*(1 - 0.3) * (2513.08314*plant_capacity_mtpy**0.63325) # Optimistic ballpark estimate of 30% reduction in cooling
       oxygen_market_price = 0.03              # $/kgO2
    else:
        capex_h2_preheating = model_year_CEPCI/equation_year_CEPCI*45.69123*plant_capacity_mtpy**0.86564
        capex_cooling_tower = model_year_CEPCI/equation_year_CEPCI*2513.08314*plant_capacity_mtpy**0.63325
        oxygen_market_price = 0 # $/kgO2
    excess_oxygen       = 395               # excess kg O2/metric tonne of steel
    capex_piping = model_year_CEPCI/equation_year_CEPCI*11815.72718*plant_capacity_mtpy**0.59983
    capex_elec_instr = model_year_CEPCI/equation_year_CEPCI*7877.15146*plant_capacity_mtpy**0.59983
    capex_buildings_storage_water = model_year_CEPCI/equation_year_CEPCI*1097.81876*plant_capacity_mtpy**0.8
    capex_misc = model_year_CEPCI/equation_year_CEPCI*7877.1546*plant_capacity_mtpy**0.59983
    
    total_plant_cost = capex_eaf_casting + capex_shaft_furnace + capex_oxygen_supply\
                     + capex_h2_preheating + capex_cooling_tower + capex_piping\
                     + capex_elec_instr + capex_buildings_storage_water + capex_misc
                     
    
    #-------------------------------Fixed O&M Costs------------------------------
    
    labor_cost_annual_operation = 69375996.9*((plant_capacity_mtpy/365*1000)**0.25242)\
                                /((1162077/365*1000)**0.25242)
    labor_cost_maintenance = 0.00863*total_plant_cost
    labor_cost_admin_support = 0.25*(labor_cost_annual_operation + labor_cost_maintenance)
    
    property_tax_insurance = 0.02*total_plant_cost
    
    total_fixed_operating_cost = labor_cost_annual_operation + labor_cost_maintenance\
                               + labor_cost_admin_support + property_tax_insurance
    
    #-------------------------- Feedstock and Waste Costs -------------------------
    
    maintenance_materials_unitcost = 7.72       # $/metric tonne of annual steel slab production at real CF
    raw_water_unitcost = 0.59289                # $/metric tonne of raw water
    lime_unitcost = lime_unitcost                         # $/metric tonne of lime
    carbon_unitcost = carbon_unitcost                    # $/metric tonne of Carbon
    slag_disposal_unitcost = 37.63              # $ metric tonne of Slag
    iron_ore_pellet_unitcost = iron_ore_pellet_unitcost          # $/metric tone of Ore 
    
    # ---------------Feedstock Consumtion and Waste/Emissions Production-----------
    
    iron_ore_consumption = 1.62927              # metric tonnes of iron ore/metric tonne of steel production
    raw_water_consumption = 0.80367             # metric tonnes of raw water/metric tonne of steel production
    lime_consumption = 0.01812                  # metric tonnes of lime/metric tonne of steel production
    carbon_consumption = 0.0538                 # metric tonnes of carbon/metric tonne of steel production
    hydrogen_consumption = 0.06596              # metric tonnes of hydrogen/metric tonne of steel production
    natural_gas_consumption = 0.71657           # GJ-LHV/metric tonne of steel production
    electricity_consumption = 0.5502            # MWh/metric tonne of steel production
    
    co2_fuel_emissions = 0.03929                # metric tonnes of CO2/metric tonne of steel production
    co2_carbon_emissions = 0.17466              # metric tonnes of CO2/metridc tonne of steel production
    slag_production = 0.17433                   # metric tonnes of slag/metric tonne of steel production
    surface_water_discharge = 0.42113           # metridc tonnes of surface water discharge/metridc tonne of steel production
    
    #---------------------- Owner's (Installation) Costs --------------------------
    labor_cost_fivemonth = 5/12*(labor_cost_annual_operation + labor_cost_maintenance \
                               + labor_cost_admin_support)
    
    maintenance_materials_onemonth = maintenance_materials_unitcost*plant_capacity_mtpy/12
    non_fuel_consumables_onemonth = plant_capacity_mtpy*(raw_water_consumption*raw_water_unitcost\
                                  + lime_consumption*lime_unitcost + carbon_consumption*carbon_unitcost\
                                  + iron_ore_consumption*iron_ore_pellet_unitcost)/12
        
    waste_disposal_onemonth = plant_capacity_mtpy*slag_disposal_unitcost*slag_production/12
    
    one_month_energy_cost_25percent = 0.25*plant_capacity_mtpy*(hydrogen_consumption*levelized_cost_of_hydrogen*1000\
                                    + natural_gas_consumption*natural_gas_cost/1.05505585\
                                    + electricity_consumption*electricity_cost)/12
    two_percent_tpc = 0.02*total_plant_cost
    
    fuel_consumables_60day_supply_cost = plant_capacity_mtpy*(raw_water_consumption*raw_water_unitcost\
                                  + lime_consumption*lime_unitcost + carbon_consumption*carbon_unitcost\
                                  + iron_ore_consumption*iron_ore_pellet_unitcost)/365*60
        
    spare_parts_cost = 0.005*total_plant_cost
    
    land_cost = 0.775*plant_capacity_mtpy
    misc_owners_costs = 0.15*total_plant_cost
    
    installation_cost = labor_cost_fivemonth + two_percent_tpc\
                       + fuel_consumables_60day_supply_cost + spare_parts_cost\
                       + misc_owners_costs
                       
    #total_overnight_capital_cost = total_plant_cost + total_owners_cost

    financial_assumptions = pd.read_csv('H2_Analysis/financial_inputs.csv',index_col=None,header=0)
    financial_assumptions.set_index(["Parameter"], inplace = True)
    financial_assumptions = financial_assumptions['Hydrogen/Steel/Ammonia']

    # Set up ProFAST
    pf = ProFAST.ProFAST()
    
    install_years = 3
    analysis_start = list(grid_prices_interpolated_USDperMWh.keys())[0] - install_years

    # Fill these in - can have most of them as 0 also
    gen_inflation = 0.00
    pf.set_params('commodity',{"name":'Steel',"unit":"metric tonnes","initial price":1000,"escalation":gen_inflation})
    pf.set_params('capacity',plant_capacity_mtpy/365) #units/day
    pf.set_params('maintenance',{"value":0,"escalation":gen_inflation})
    pf.set_params('analysis start year',analysis_start)
    pf.set_params('operating life',plant_life)
    pf.set_params('installation months',12*install_years)
    pf.set_params('installation cost',{"value":installation_cost,"depr type":"Straight line","depr period":4,"depreciable":False})
    pf.set_params('non depr assets',land_cost)
    pf.set_params('end of proj sale non depr assets',land_cost*(1+gen_inflation)**plant_life)
    pf.set_params('demand rampup',5.3)
    pf.set_params('long term utilization',plant_capacity_factor)
    pf.set_params('credit card fees',0)
    pf.set_params('sales tax',0) 
    pf.set_params('license and permit',{'value':00,'escalation':gen_inflation})
    pf.set_params('rent',{'value':0,'escalation':gen_inflation})
    pf.set_params('property tax and insurance',0)
    pf.set_params('admin expense',0)
    pf.set_params('total income tax rate',financial_assumptions['total income tax rate'])
    pf.set_params('capital gains tax rate',financial_assumptions['capital gains tax rate'])
    pf.set_params('sell undepreciated cap',True)
    pf.set_params('tax losses monetized',True)
    pf.set_params('general inflation rate',gen_inflation)
    pf.set_params('leverage after tax nominal discount rate',financial_assumptions['leverage after tax nominal discount rate'])
    pf.set_params('debt equity ratio of initial financing',financial_assumptions['debt equity ratio of initial financing'])
    pf.set_params('debt type','Revolving debt')
    pf.set_params('debt interest rate',financial_assumptions['debt interest rate'])
    pf.set_params('cash onhand',1)
    
    #----------------------------------- Add capital items to ProFAST ----------------
    pf.add_capital_item(name="EAF & Casting",cost=capex_eaf_casting,depr_type="MACRS",depr_period=7,refurb=[0])
    pf.add_capital_item(name="Shaft Furnace",cost=capex_shaft_furnace,depr_type="MACRS",depr_period=7,refurb=[0])
    pf.add_capital_item(name="Oxygen Supply",cost=capex_oxygen_supply,depr_type="MACRS",depr_period=7,refurb=[0])
    pf.add_capital_item(name="H2 Pre-heating",cost=capex_h2_preheating,depr_type="MACRS",depr_period=7,refurb=[0])
    pf.add_capital_item(name="Cooling Tower",cost=capex_cooling_tower,depr_type="MACRS",depr_period=7,refurb=[0])
    pf.add_capital_item(name="Piping",cost=capex_piping,depr_type="MACRS",depr_period=7,refurb=[0])
    pf.add_capital_item(name="Electrical & Instrumentation",cost=capex_elec_instr,depr_type="MACRS",depr_period=7,refurb=[0])
    pf.add_capital_item(name="Buildings, Storage, Water Service",cost=capex_buildings_storage_water,depr_type="MACRS",depr_period=7,refurb=[0])
    pf.add_capital_item(name="Other Miscellaneous Costs",cost=capex_misc,depr_type="MACRS",depr_period=7,refurb=[0])

    total_capex = capex_eaf_casting+capex_shaft_furnace+capex_oxygen_supply+capex_h2_preheating+capex_cooling_tower+capex_piping+capex_elec_instr+capex_buildings_storage_water+capex_misc
    
    #-------------------------------------- Add fixed costs--------------------------------
    pf.add_fixed_cost(name="Annual Operating Labor Cost",usage=1,unit='$/year',cost=labor_cost_annual_operation,escalation=gen_inflation)
    pf.add_fixed_cost(name="Maintenance Labor Cost",usage=1,unit='$/year',cost=labor_cost_maintenance,escalation=gen_inflation)
    pf.add_fixed_cost(name="Administrative & Support Labor Cost",usage=1,unit='$/year',cost=labor_cost_admin_support,escalation=gen_inflation)
    pf.add_fixed_cost(name="Property tax and insurance",usage=1,unit='$/year',cost=0.02*total_plant_cost,escalation=0.0) 
    # Putting property tax and insurance here to zero out depcreciation/escalation. Could instead put it in set_params if
    # we think that is more accurate
    
    #---------------------- Add feedstocks, note the various cost options-------------------
    pf.add_feedstock(name='Maintenance Materials',usage=1.0,unit='Units per metric tonne of steel',cost=maintenance_materials_unitcost,escalation=gen_inflation)
    pf.add_feedstock(name='Raw Water Withdrawal',usage=raw_water_consumption,unit='metric tonnes of water per metric tonne of steel',cost=raw_water_unitcost,escalation=gen_inflation)
    pf.add_feedstock(name='Lime',usage=lime_consumption,unit='metric tonnes of lime per metric tonne of steel',cost=lime_unitcost,escalation=gen_inflation)
    pf.add_feedstock(name='Carbon',usage=carbon_consumption,unit='metric tonnes of carbon per metric tonne of steel',cost=carbon_unitcost,escalation=gen_inflation)
    pf.add_feedstock(name='Iron Ore',usage=iron_ore_consumption,unit='metric tonnes of iron ore per metric tonne of steel',cost=iron_ore_pellet_unitcost,escalation=gen_inflation)
    pf.add_feedstock(name='Hydrogen',usage=hydrogen_consumption,unit='metric tonnes of hydrogen per metric tonne of steel',cost=levelized_cost_of_hydrogen*1000,escalation=gen_inflation)
    pf.add_feedstock(name='Natural Gas',usage=natural_gas_consumption,unit='GJ-LHV per metric tonne of steel',cost=natural_gas_cost/1.05505585,escalation=gen_inflation)
    pf.add_feedstock(name='Electricity',usage=electricity_consumption,unit='MWh per metric tonne of steel',cost=grid_prices_interpolated_USDperMWh,escalation=gen_inflation)
    pf.add_feedstock(name='Slag Disposal',usage=slag_production,unit='metric tonnes of slag per metric tonne of steel',cost=slag_disposal_unitcost,escalation=gen_inflation)

    pf.add_coproduct( name = 'Oxygen sales', usage = excess_oxygen, unit='kg O2 per metric tonne of steel', cost = oxygen_market_price, escalation=gen_inflation)
# Not sure if ProFAST can work with negative cost i.e., revenues so, will add the reduction at the end
    # if o2_heat_integration == 1:
    #     pf.addfeedstock(name='Oxygen Sales',usage=excess_oxygen,unit='kilograms of oxygen per metric tonne of steel',cost=-oxygen_market_price,escalation=gen_inflation)
    #------------------------------ Sovle for breakeven price ---------------------------
    
    sol = pf.solve_price()
    
    summary = pf.summary_vals
    
    price_breakdown = pf.get_cost_breakdown()
    
    price_breakdown_eaf_casting = price_breakdown.loc[price_breakdown['Name']=='EAF & Casting','NPV'].tolist()[0]
    price_breakdown_shaft_furnace = price_breakdown.loc[price_breakdown['Name']=='Shaft Furnace','NPV'].tolist()[0]  
    price_breakdown_oxygen_supply = price_breakdown.loc[price_breakdown['Name']=='Oxygen Supply','NPV'].tolist()[0] 
    price_breakdown_h2_preheating = price_breakdown.loc[price_breakdown['Name']=='H2 Pre-heating','NPV'].tolist()[0]     
    price_breakdown_cooling_tower = price_breakdown.loc[price_breakdown['Name']=='Cooling Tower','NPV'].tolist()[0] 
    price_breakdown_piping = price_breakdown.loc[price_breakdown['Name']=='Piping','NPV'].tolist()[0] 
    price_breakdown_elec_instr = price_breakdown.loc[price_breakdown['Name']=='Electrical & Instrumentation','NPV'].tolist()[0] 
    price_breakdown_buildings_storage_water = price_breakdown.loc[price_breakdown['Name']=='Buildings, Storage, Water Service','NPV'].tolist()[0]  
    price_breakdown_misc = price_breakdown.loc[price_breakdown['Name']=='Other Miscellaneous Costs','NPV'].tolist()[0]
    price_breakdown_installation = price_breakdown.loc[price_breakdown['Name']=='Installation cost','NPV'].tolist()[0]
 
    
    price_breakdown_labor_cost_annual = price_breakdown.loc[price_breakdown['Name']=='Annual Operating Labor Cost','NPV'].tolist()[0]  
    price_breakdown_labor_cost_maintenance = price_breakdown.loc[price_breakdown['Name']=='Maintenance Labor Cost','NPV'].tolist()[0]  
    price_breakdown_labor_cost_admin_support = price_breakdown.loc[price_breakdown['Name']=='Administrative & Support Labor Cost','NPV'].tolist()[0] 
    #price_breakdown_proptax_ins = price_breakdown.loc[price_breakdown['Name']=='Property tax and insurance','NPV'].tolist()[0]
    
    price_breakdown_maintenance_materials = price_breakdown.loc[price_breakdown['Name']=='Maintenance Materials','NPV'].tolist()[0]  
    price_breakdown_water_withdrawal = price_breakdown.loc[price_breakdown['Name']=='Raw Water Withdrawal','NPV'].tolist()[0] 
    price_breakdown_lime = price_breakdown.loc[price_breakdown['Name']=='Lime','NPV'].tolist()[0]
    price_breakdown_carbon = price_breakdown.loc[price_breakdown['Name']=='Carbon','NPV'].tolist()[0]
    price_breakdown_iron_ore = price_breakdown.loc[price_breakdown['Name']=='Iron Ore','NPV'].tolist()[0]
    if levelized_cost_of_hydrogen < 0:
        price_breakdown_hydrogen = -1*price_breakdown.loc[price_breakdown['Name']=='Hydrogen','NPV'].tolist()[0]
    else:
        price_breakdown_hydrogen = price_breakdown.loc[price_breakdown['Name']=='Hydrogen','NPV'].tolist()[0]
    price_breakdown_natural_gas = price_breakdown.loc[price_breakdown['Name']=='Natural Gas','NPV'].tolist()[0]
    price_breakdown_electricity = price_breakdown.loc[price_breakdown['Name']=='Electricity','NPV'].tolist()[0]
    price_breakdown_slag =  price_breakdown.loc[price_breakdown['Name']=='Slag Disposal','NPV'].tolist()[0]
    price_breakdown_taxes = price_breakdown.loc[price_breakdown['Name']=='Income taxes payable','NPV'].tolist()[0]\
        - price_breakdown.loc[price_breakdown['Name'] == 'Monetized tax losses','NPV'].tolist()[0]\
    
    if o2_heat_integration == 1:
        price_breakdown_O2sales =  price_breakdown.loc[price_breakdown['Name']=='Oxygen sales','NPV'].tolist()[0]    
    else:
        price_breakdown_O2sales = 0
        
    if gen_inflation > 0:
        price_breakdown_taxes = price_breakdown_taxes + price_breakdown.loc[price_breakdown['Name']=='Capital gains taxes payable','NPV'].tolist()[0]
        
    # price_breakdown_financial = price_breakdown.loc[price_breakdown['Name']=='Non-depreciable assets','NPV'].tolist()[0]\
    #     + price_breakdown.loc[price_breakdown['Name']=='Cash on hand reserve','NPV'].tolist()[0]\
    #     + price_breakdown.loc[price_breakdown['Name']=='Property tax and insurance','NPV'].tolist()[0]\
    #     + price_breakdown.loc[price_breakdown['Name']=='Repayment of debt','NPV'].tolist()[0]\
    #     + price_breakdown.loc[price_breakdown['Name']=='Interest expense','NPV'].tolist()[0]\
    #     + price_breakdown.loc[price_breakdown['Name']=='Dividends paid','NPV'].tolist()[0]\
    #     - price_breakdown.loc[price_breakdown['Name']=='Sale of non-depreciable assets','NPV'].tolist()[0]\
    #     - price_breakdown.loc[price_breakdown['Name']=='Cash on hand recovery','NPV'].tolist()[0]\
    #     - price_breakdown.loc[price_breakdown['Name']=='Inflow of debt','NPV'].tolist()[0]\
    #     - price_breakdown.loc[price_breakdown['Name']=='Inflow of equity','NPV'].tolist()[0]

    # Calculate financial expense associated with equipment
    price_breakdown_financial_equipment = price_breakdown.loc[price_breakdown['Name']=='Repayment of debt','NPV'].tolist()[0]\
        + price_breakdown.loc[price_breakdown['Name']=='Interest expense','NPV'].tolist()[0]\
        + price_breakdown.loc[price_breakdown['Name']=='Dividends paid','NPV'].tolist()[0]\
        - price_breakdown.loc[price_breakdown['Name']=='Inflow of debt','NPV'].tolist()[0]\
        - price_breakdown.loc[price_breakdown['Name']=='Inflow of equity','NPV'].tolist()[0]    
        
    # Calculate remaining financial expenses
    price_breakdown_financial_remaining = price_breakdown.loc[price_breakdown['Name']=='Non-depreciable assets','NPV'].tolist()[0]\
        + price_breakdown.loc[price_breakdown['Name']=='Cash on hand reserve','NPV'].tolist()[0]\
        + price_breakdown.loc[price_breakdown['Name']=='Property tax and insurance','NPV'].tolist()[0]\
        - price_breakdown.loc[price_breakdown['Name']=='Sale of non-depreciable assets','NPV'].tolist()[0]\
        - price_breakdown.loc[price_breakdown['Name']=='Cash on hand recovery','NPV'].tolist()[0]
    

    price_breakdown_check = price_breakdown_eaf_casting+price_breakdown_shaft_furnace+price_breakdown_oxygen_supply+price_breakdown_h2_preheating\
            +price_breakdown_cooling_tower+price_breakdown_piping+price_breakdown_elec_instr+price_breakdown_buildings_storage_water+price_breakdown_misc\
            +price_breakdown_installation+price_breakdown_labor_cost_annual+price_breakdown_labor_cost_maintenance+price_breakdown_labor_cost_admin_support\
            +price_breakdown_maintenance_materials+price_breakdown_water_withdrawal+price_breakdown_lime+price_breakdown_carbon+price_breakdown_iron_ore\
            +price_breakdown_hydrogen+price_breakdown_natural_gas+price_breakdown_electricity+price_breakdown_slag+price_breakdown_taxes+price_breakdown_financial_equipment\
            +price_breakdown_financial_remaining+price_breakdown_O2sales    # a neater way to implement is add to price_breakdowns but I am not sure if ProFAST can handle negative costs
 
        
    bos_savings =  (price_breakdown_labor_cost_admin_support) * 0.3
    steel_price_breakdown = {'Steel price: EAF and Casting CAPEX ($/tonne)':price_breakdown_eaf_casting,'Steel price: Shaft Furnace CAPEX ($/tonne)':price_breakdown_shaft_furnace,\
                             'Steel price: Oxygen Supply CAPEX ($/tonne)':price_breakdown_oxygen_supply,'Steel price: H2 Pre-heating CAPEX ($/tonne)':price_breakdown_h2_preheating,\
                          'Steel price: Cooling Tower CAPEX ($/tonne)':price_breakdown_cooling_tower,'Steel price: Piping CAPEX ($/tonne)':price_breakdown_piping,\
                          'Steel price: Electrical & Instrumentation ($/tonne)':price_breakdown_elec_instr,'Steel price: Buildings, Storage, Water Service CAPEX ($/tonne)':price_breakdown_buildings_storage_water,\
                          'Steel price: Miscellaneous CAPEX ($/tonne)':price_breakdown_misc,'Steel price: Annual Operating Labor Cost ($/tonne)':price_breakdown_labor_cost_annual,\
                          'Steel price: Maintenance Labor Cost ($/tonne)':price_breakdown_labor_cost_maintenance,'Steel price: Administrative & Support Labor Cost ($/tonne)':price_breakdown_labor_cost_admin_support,\
                          'Steel price: Installation Cost ($/tonne)':price_breakdown_installation,'Steel price: Maintenance Materials ($/tonne)':price_breakdown_maintenance_materials,\
                          'Steel price: Raw Water Withdrawal ($/tonne)':price_breakdown_water_withdrawal,'Steel price: Lime ($/tonne)':price_breakdown_lime,\
                          'Steel price: Carbon ($/tonne)':price_breakdown_carbon,'Steel price: Iron Ore ($/tonne)':price_breakdown_iron_ore,\
                          'Steel price: Hydrogen ($/tonne)':price_breakdown_hydrogen,'Steel price: Natural gas ($/tonne)':price_breakdown_natural_gas,\
                          'Steel price: Electricity ($/tonne)':price_breakdown_electricity,'Steel price: Slag Disposal ($/tonne)':price_breakdown_slag,\
                          'Steel price: Taxes ($/tonne)':price_breakdown_taxes,'Steel price: Equipment Financing ($/tonne)':price_breakdown_financial_equipment,\
                          'Steel price: Remaining Financial ($/tonne)':price_breakdown_financial_remaining,'Steel price: Oxygen sales ($/tonne)': price_breakdown_O2sales,\
                          'Steel price: Total ($/tonne)':price_breakdown_check, '(-) Steel price: BOS savings ($/tonne)': bos_savings}
    
    price_breakdown = price_breakdown.drop(columns=['index','Amount'])

    return(sol,summary,price_breakdown,steel_production_mtpy,steel_price_breakdown,total_capex)









from greenheart.simulation.technologies.hydrogen.electrolysis.H2_cost_model import basic_H2_cost_model
from pytest import approx
import numpy as np

class TestBasicH2Costs():

    electrolyzer_size_mw = 100
    h2_annual_output = 500
    nturbines = 10
    electrical_generation_timeseries = electrolyzer_size_mw*(np.sin(range(0,500)))*0.5 + electrolyzer_size_mw*0.5

    per_turb_electrolyzer_size_mw = electrolyzer_size_mw/nturbines
    per_turb_h2_annual_output = h2_annual_output/nturbines
    per_turb_electrical_generation_timeseries = electrical_generation_timeseries/nturbines

    elec_capex = 600 # $/kW
    time_between_replacement = 80000 #hours
    useful_life = 30 # years
    atb_year = 2022

    def test_on_turbine_capex(self):
        
        cf_h2_annuals, per_turb_electrolyzer_total_capital_cost, per_turb_electrolyzer_OM_cost, per_turb_electrolyzer_capex_kw, \
        time_between_replacement, h2_tax_credit, h2_itc = \
            basic_H2_cost_model(self.elec_capex, self.time_between_replacement, self.per_turb_electrolyzer_size_mw, self.useful_life, self.atb_year,
            self.per_turb_electrical_generation_timeseries, self.per_turb_h2_annual_output, 0.0, 0.0, include_refurb_in_opex=False, offshore=1)

        electrolyzer_total_capital_cost = per_turb_electrolyzer_total_capital_cost*self.nturbines
        electrolyzer_OM_cost = per_turb_electrolyzer_OM_cost*self.nturbines

        assert electrolyzer_total_capital_cost == approx(127698560.0)

    def test_on_platform_capex(self):
        
        cf_h2_annuals, electrolyzer_total_capital_cost, electrolyzer_OM_cost, electrolyzer_capex_kw, \
        time_between_replacement, h2_tax_credit, h2_itc = \
            basic_H2_cost_model(self.elec_capex, self.time_between_replacement, self.electrolyzer_size_mw, self.useful_life, self.atb_year,
            self.electrical_generation_timeseries, self.h2_annual_output, 0.0, 0.0, include_refurb_in_opex=False, offshore=1)

        assert electrolyzer_total_capital_cost == approx(125448560.0)

    def test_on_land_capex(self):
        
        cf_h2_annuals, per_turb_electrolyzer_total_capital_cost, per_turb_electrolyzer_OM_cost, per_turb_electrolyzer_capex_kw, \
        time_between_replacement, h2_tax_credit, h2_itc = \
            basic_H2_cost_model(self.elec_capex, self.time_between_replacement, self.per_turb_electrolyzer_size_mw, self.useful_life, self.atb_year,
            self.per_turb_electrical_generation_timeseries, self.per_turb_h2_annual_output, 0.0, 0.0, include_refurb_in_opex=False, offshore=0)

        electrolyzer_total_capital_cost = per_turb_electrolyzer_total_capital_cost*self.nturbines
        electrolyzer_OM_cost = per_turb_electrolyzer_OM_cost*self.nturbines

        assert electrolyzer_total_capital_cost == approx(116077280.00000003)

    def test_on_turbine_opex(self):
        
        cf_h2_annuals, per_turb_electrolyzer_total_capital_cost, per_turb_electrolyzer_OM_cost, per_turb_electrolyzer_capex_kw, \
        time_between_replacement, h2_tax_credit, h2_itc = \
            basic_H2_cost_model(self.elec_capex, self.time_between_replacement, self.per_turb_electrolyzer_size_mw, self.useful_life, self.atb_year,
            self.per_turb_electrical_generation_timeseries, self.per_turb_h2_annual_output, 0.0, 0.0, include_refurb_in_opex=False, offshore=1)

        electrolyzer_total_capital_cost = per_turb_electrolyzer_total_capital_cost*self.nturbines
        electrolyzer_OM_cost = per_turb_electrolyzer_OM_cost*self.nturbines

        assert electrolyzer_OM_cost == approx(1377207.4599629682)

    def test_on_platform_opex(self):
        
        cf_h2_annuals, electrolyzer_total_capital_cost, electrolyzer_OM_cost, electrolyzer_capex_kw, \
        time_between_replacement, h2_tax_credit, h2_itc = \
            basic_H2_cost_model(self.elec_capex, self.time_between_replacement, self.electrolyzer_size_mw, self.useful_life, self.atb_year,
            self.electrical_generation_timeseries, self.h2_annual_output, 0.0, 0.0, include_refurb_in_opex=False, offshore=1)

        assert electrolyzer_OM_cost == approx(1864249.9310054395)

    def test_on_land_opex(self):
        
        cf_h2_annuals, per_turb_electrolyzer_total_capital_cost, per_turb_electrolyzer_OM_cost, per_turb_electrolyzer_capex_kw, \
        time_between_replacement, h2_tax_credit, h2_itc = \
            basic_H2_cost_model(self.elec_capex, self.time_between_replacement, self.per_turb_electrolyzer_size_mw, self.useful_life, self.atb_year,
            self.per_turb_electrical_generation_timeseries, self.per_turb_h2_annual_output, 0.0, 0.0, include_refurb_in_opex=False, offshore=0)

        electrolyzer_total_capital_cost = per_turb_electrolyzer_total_capital_cost*self.nturbines
        electrolyzer_OM_cost = per_turb_electrolyzer_OM_cost*self.nturbines

        assert electrolyzer_OM_cost == approx(1254447.4599629682)

if __name__ == "__main__":
    test_set = TestBasicH2Costs()



import math
import pandas as pd
import numpy as np


def H2AModel(cap_factor, avg_daily_H2_production, hydrogen_annual_output, h2a_for_hopp=True, force_system_size=True,
             forced_system_size=50, force_electrolyzer_cost=False, forced_electrolyzer_cost_kw=200, useful_life = 30):

    # ---------------------------------------------------H2A PROCESS FLOW----------------------------------------------------------#


    current_density = 2  # A/cm^2
    voltage = 1.9  # V/cell
    operating_temp = 80  # C
    H2_outlet_pressure = 450  # psi
    cell_active_area = 450  # cm^2
    cellperstack = 150  # cells
    degradation_rate = 1.5  # mV/1000 hrs
    stack_life = 7  # years
    hours_per_stack_life = stack_life * 365 * 24 * cap_factor  # hrs/life
    degradation_rate_Vperlife = hours_per_stack_life * degradation_rate / 1000  # V/life
    stack_degradation_oversize = 0.13  # factor
    peak_daily_production_rate = avg_daily_H2_production * (1 + stack_degradation_oversize)  # kgH2/day

    total_active_area = math.ceil(
        (avg_daily_H2_production / 2.02 * 1000 / 24 / 3600) * 2 * 96485 / current_density / (100 ** 2))  # m^2
    total_active_area_degraded = math.ceil(
        (peak_daily_production_rate / 2.02 * 1000 / 24 / 3600) * 2 * 96485 / current_density / (100 ** 2))  # m^2

    stack_electrical_usage = 50.4  # kWh/kgH2
    BoP_electrical_usage = 5.1  # kWh/kgH2
    total_system_electrical_usage = 55.5  # kWh/kg H2

    if forced_system_size:
        total_system_input = forced_system_size  # MW
        stack_input_power = (stack_electrical_usage/ total_system_electrical_usage) * forced_system_size  # MW
    else:
        total_system_input = total_system_electrical_usage / 24 * peak_daily_production_rate / 1000  # MW
        stack_input_power = stack_electrical_usage / 24 * peak_daily_production_rate / 1000  # MW

    process_water_flowrate = 3.78

    system_unit_cost = forced_electrolyzer_cost_kw #1.3 * 300/342 # $/cm^2
    stack_system_cost = system_unit_cost / (current_density * voltage) * 1000  # $/kW
    mechanical_BoP_unit_cost = 76  # kWhH2/day
    mechanical_BoP_cost = 76 * peak_daily_production_rate / stack_input_power / 1000  # $/kW
    electrical_BoP_cost = 82  # $/kW
    total_system_cost_perkW = stack_system_cost + mechanical_BoP_cost + electrical_BoP_cost  # $/kW
    total_system_cost_perkW = total_system_cost_perkW
    if force_electrolyzer_cost:
        total_system_cost = forced_electrolyzer_cost_kw * stack_input_power * 1000
        # print("Can confirm I did this", total_system_cost)
    else:
        total_system_cost = total_system_cost_perkW * stack_input_power * 1000  # $


    # -------------------------------------------------CAPITAL COST--------------------------------------------------------------#


    gdpdef = {'Year': [2015, 2016, 2017, 2018, 2019, 2020], 'CEPCI': [104.031, 104.865, 107.010, 109.237, 111.424,
                                                                      113.415]}  # GDPDEF (2012=100), https://fred.stlouisfed.org/series/GDPDEF/
    CEPCI = pd.DataFrame(data=gdpdef)  # Deflator Table

    pci = {'Year': [2015, 2016, 2017, 2018, 2019, 2020],
           'PCI': [556.8, 541.7, 567.5, 603.1, 607.5, 610]}  # plant cost index, Chemical Engineering Magazine
    CPI = pd.DataFrame(data=pci)

    baseline_plant_design_capacity = avg_daily_H2_production
    basis_year_for_capital_cost = 2016
    current_year_for_capital_cost = 2016
    CEPCI_inflator = int((CEPCI.loc[CEPCI['Year'] == current_year_for_capital_cost, 'CEPCI']).iloc[0]) / int(
        (CEPCI.loc[CEPCI['Year'] == basis_year_for_capital_cost, 'CEPCI']).iloc[0])
    consumer_price_inflator = int((CPI.loc[CPI['Year'] == current_year_for_capital_cost, 'PCI']).iloc[0]) / int(
        (CPI.loc[CPI['Year'] == basis_year_for_capital_cost, 'PCI']).iloc[0])  # lookup

    # --------------------------CAPITAL INVESTMENT---------------------------------#
    # ----Inputs required in basis year (2016$)----#

    baseline_uninstalled_stack_capital_cost = CEPCI_inflator * consumer_price_inflator * system_unit_cost * total_active_area_degraded * 100 ** 2  # ($2016)
    stack_installation_factor = 1.12
    baseline_installed_stack_capital_cost = stack_installation_factor * baseline_uninstalled_stack_capital_cost

    baseline_uninstalled_mechanical_BoP_cost = CEPCI_inflator * consumer_price_inflator * mechanical_BoP_unit_cost * peak_daily_production_rate  # ($2016)
    mechanical_BoP_installation_factor = 1
    baseline_installed_mechanical_BoP_cost = mechanical_BoP_installation_factor * baseline_uninstalled_mechanical_BoP_cost

    baseline_uninstalled_electrical_BoP_cost = CEPCI_inflator * consumer_price_inflator * electrical_BoP_cost * stack_input_power * 1000  # ($2016)
    electrical_BoP_installation_factor = 1.12
    baseline_installed_electrical_BoP_cost = electrical_BoP_installation_factor * baseline_uninstalled_electrical_BoP_cost

    baseline_total_installed_cost = baseline_installed_stack_capital_cost + baseline_installed_mechanical_BoP_cost + baseline_installed_electrical_BoP_cost

    # ------------------------------------------------PLANT SCALING-------------------------------------------------------------------#

    scale_ratio = 1  # ratio of new design capacity to baseline design capacity (linear scaling)
    scale_factor = 1  # rato of total scaled installed capital cost to total baseline installed capital cost (exponential scaling)
    default_scaling_factor_exponent = 1  # discrepancy
    lower_limit_for_scaling_capacity = 20000  # kgH2/day
    upper_limit_for_scaling_capacity = 200000  # kgH2/day

    scaled_uninstalled_stack_capital_cost = baseline_uninstalled_stack_capital_cost ** default_scaling_factor_exponent
    scaled_installed_stack_capital_cost = scaled_uninstalled_stack_capital_cost * stack_installation_factor

    scaled_uninstalled_mechanical_BoP_cost = baseline_uninstalled_mechanical_BoP_cost ** default_scaling_factor_exponent
    scaled_installed_mechanical_BoP_cost = scaled_uninstalled_mechanical_BoP_cost * mechanical_BoP_installation_factor

    scaled_uninstalled_electrical_BoP_cost = baseline_uninstalled_electrical_BoP_cost ** default_scaling_factor_exponent
    scaled_installed_electrical_BoP_cost = scaled_uninstalled_electrical_BoP_cost * electrical_BoP_installation_factor
    
    #TODO: Rectify this calculation. This is a shortcut that is pretty SUS.
    scaled_total_installed_cost = forced_electrolyzer_cost_kw * stack_input_power * 1000 #scaled_installed_stack_capital_cost + scaled_installed_mechanical_BoP_cost + scaled_installed_electrical_BoP_cost

    # -------------------------------------------------------H2A INPUT-------------------------------------------------------------#

    # --------------------------TECHNICAL OPERATING PARAMETERS AND SPECIFICATIONS---------------------------#

    operating_capacity_factor = cap_factor
    plant_design_capacity = peak_daily_production_rate  # kgH2/day
    plant_daily_output = plant_design_capacity  # kg/day
    plant_annual_output = hydrogen_annual_output  # kg/year

    # -----------------------------------FINANCIAL INPUT VALUES---------------------------------------------#

    reference_year = 2016
    assumed_startup_year = 2015
    basis_year = 2016
    length_of_construction_period = 1  # year
    startup_time = length_of_construction_period
    percent_capital_spent_year_1 = 100 / 100  # percent
    percent_Capital_Spent_year_2 = 0 / 100  # percent
    percent_Capital_Spent_year_3 = 0 / 100  # percent
    percent_Capital_Spent_year_4 = 0 / 100  # percent
    plant_life = useful_life  # years
    analysis_period = useful_life  # years
    depreciation_schedule_length = 20  # years
    depreciation_type = 'MACRS'
    percent_equity_financing = 40 / 100  # percent
    debt_financing = 60 / 100  # percent
    interest_rate_on_debt = 3.7 / 100  # percent
    debt_period = 'Constant Debt'
    percent_fixed_operating_cost_during_startup = 75 / 100  # percent
    percent_revenue_during_startup = 50 / 100  # percent
    percent_variable_operating_cost_during_startup = 75 / 100  # percent
    decom_pct = 10 / 100  # percent of depreciable capital investment
    salvage_pct = 10 / 100  # percent of total capital investment
    inflation_rate = 1.9 / 100  # percent
    inflation_factor = (1 + inflation_rate) ** (assumed_startup_year - reference_year)
    after_tax_real_IRR = 8 / 100  # percent
    nominal_IRR = ((1 + after_tax_real_IRR) * (1 + inflation_rate)) - 1
    state_taxes = 6 / 100  # percent
    federal_taxes = 21 / 100  # percent
    total_tax_rate = federal_taxes + state_taxes * (1 - federal_taxes)  # percent
    working_capital = 15 / 100  # percent of yearly change in operating cost
    unplanned_replacement_cost_factor = 0.005
    tax_credit = 0

    # ------------------------Energy Feedstock, Ulilities, and Byproducts---------------------------------------#

    price_table_reference = 'AEO_2017_Reference_Case'

    # Add Feedstock Info
    feedstock_type = '--------'
    feedstock_price_conversion_factor = 0  # GJ/kWh
    feedstock_price_in_startup_year = 0  # ($2016)/kWh (LookUp)
    feedstock_usage = 55.5  # kWh/kgH2 (LookUp)
    total_energy_feedstock_unitcost = 0.00
    feedstock_cost_in_startup_year = plant_annual_output * feedstock_usage * feedstock_price_in_startup_year

    total_energy_feedstock_cost = feedstock_cost_in_startup_year  # Add all feedstock costs in startup year

    # Add Utilities Info
    utilities_type = 'Industrial Electricity'
    electricity_LHV = 0.0036  # GJ/kWh
    utilities_price_in_startup_year = 0.00  # ($2016)/kWh (LookUp)
    electricity_usage = 55.5  # kWh/kgH2 (LookUp)
    utilities_cost_in_startup_year = plant_annual_output * electricity_usage * utilities_price_in_startup_year

    total_energy_utilities_cost = utilities_cost_in_startup_year  # Add all utilities costs in startup year

    # Add ByProducts Info
    byproduct_type = '--------------'
    byproduct_price_conversion_factor = 0  # GJ/kWh
    byproduct_price_in_startup_year = 0  # ($2016)/kWh (LookUp)
    byproduct_production = 0  # kWh/kgH2 (LookUp)
    total_byproduct_unitprice = 0
    byproduct_income_in_startup_year = plant_annual_output * byproduct_production * byproduct_price_in_startup_year

    total_energy_byproduct_credit = byproduct_income_in_startup_year  # Add all byproduct credits in startup year

    # --------------------------------------Capital Costs--------------------------------------------------------#

    H2A_total_direct_capital_cost = int(scaled_total_installed_cost)
    cost_scaling_factor = 1  # combined plant scaling and escalation factor

    # ------------Indirect Depreciable Capital Costs---------------------#
    site_preparation = 0.02 * H2A_total_direct_capital_cost / (
            CEPCI_inflator * consumer_price_inflator) * cost_scaling_factor
    engineering_design = 0.1 * H2A_total_direct_capital_cost / (
            CEPCI_inflator * consumer_price_inflator) * cost_scaling_factor
    project_contingency = 0.15 * H2A_total_direct_capital_cost / (
            CEPCI_inflator * consumer_price_inflator) * cost_scaling_factor
    upfront_permitting_cost = 0.15 * H2A_total_direct_capital_cost / (
            CEPCI_inflator * consumer_price_inflator) * cost_scaling_factor

    total_depreciable_costs = int(
        H2A_total_direct_capital_cost + site_preparation + engineering_design + project_contingency + upfront_permitting_cost)

    # ------------Non Depreciable Capital Costs---------------------#
    cost_of_land = 50000  # ($2016)/acre
    land_required = 5  # acres
    land_cost = cost_of_land * land_required
    other_nondepreciable_cost = 0

    total_nondepreciable_costs = land_cost + other_nondepreciable_cost

    total_capital_costs = total_depreciable_costs + total_nondepreciable_costs

    # --------------------------------------Fixed Operating Costs------------------------------------------------#
    total_plant_staff = 10  # number of FTEs employed by plant
    burdened_labor_cost = 50  # including overhead ($/man-hr)
    labor_cost = total_plant_staff * burdened_labor_cost * 2080  # ($2016)/year

    GA_rate = 20  # percent labor cos (general and admin)
    GA_cost = labor_cost * (GA_rate / 100)  # $/year
    licensing_permits_fees = 0  # $/year
    propertytax_insurancerate = 2  # percent of total capital investment per year
    propertytax_insurancecost = total_capital_costs * (propertytax_insurancerate / 100)  # $/year
    rent = 0  # $/year
    material_costs_for_maintenance = 0.03 * H2A_total_direct_capital_cost / (CEPCI_inflator * consumer_price_inflator)
    production_maintenance_and_repairs = 0  # $/year
    other_fees = 0  # $/year
    other_fixed_OM_costs = 0  # $/year

    total_fixed_operating_costs = int(labor_cost + GA_cost + licensing_permits_fees + propertytax_insurancecost + rent \
                                      + material_costs_for_maintenance + production_maintenance_and_repairs + other_fees + other_fixed_OM_costs)

    # --------------------------------------Variable Operating Costs----------------------------------------------#

    # ------------------Other Material and Byproduct---------------------#

    Material_1 = 'Processed Water'  # feed
    processed_water_cost = 0.002375  # ($2016)/gal
    water_usage_per_kgH2 = 3.78  # usageperkgH2
    feedcost_in_startup_year = plant_annual_output * processed_water_cost * water_usage_per_kgH2  # lookup

    total_nonE_utility_material_costs = feedcost_in_startup_year  # Add all feed or utlity start-up year costs
    total_nonE_byproduct_credit = 0  # Add all byproduct start-up year credits

    total_feedstock_cost = total_energy_feedstock_cost
    total_utility_cost = total_energy_utilities_cost + total_nonE_utility_material_costs
    total_byproduct_credit = total_energy_byproduct_credit + total_nonE_byproduct_credit

    # -----------------Other Variable Operating Costs---------------------#

    other_variable_operating_costs = 0  # ($2016)/year (e.g. environmental surchages)
    other_material_costs = 0  # ($2016)/year
    waste_treatment_costs = 0  # ($2016)/year
    solid_waste_disposal_costs = 0  # ($2016)/year
    total_unplanned_replacement_capital_cost_factor = 0.5  # percent of total direct depreciable cost (costs/year)
    total_unplanned_replacement_capital_cost = total_depreciable_costs * (
            total_unplanned_replacement_capital_cost_factor / 100)  # ($2016)/year

    total_variable_operating_costs = (
            total_feedstock_cost + total_utility_cost + total_byproduct_credit - total_byproduct_credit +
            other_variable_operating_costs + other_material_costs + waste_treatment_costs +
            solid_waste_disposal_costs + total_unplanned_replacement_capital_cost_factor + total_unplanned_replacement_capital_cost)  # ($2016)/year

    # ---------------------------------------------------RESULTS----------------------------------------------------------#

    # --------------------------------------Hydorgen Energy Constants----------------------------------------------#

    H2_LHV_MJkg = 120.21  # MJ/kg
    H2_LHV_Btulb = 51.682  # Btu/lb
    mmBTU_to_GJ = 1.055

    # --------------------------------------DCF Calculation Inputs----------------------------------------------#

    # PROCESS
    actual_hydrogen_produced = plant_annual_output  # kg/yr
    actual_hydrogen_energy_produced_Btu = actual_hydrogen_produced * H2_LHV_MJkg / 1000 / mmBTU_to_GJ  # MMBtu(LHV)/yr
    actual_hydrogen_energy_produced_MJ = actual_hydrogen_produced * H2_LHV_MJkg

    # Additional FINANCIALS

    inflated_fixed = total_fixed_operating_costs * inflation_factor
    total_depreciable_costs_inflated = (total_capital_costs - total_nondepreciable_costs) * inflation_factor
    decom = decom_pct * total_depreciable_costs * inflation_factor
    salvage_value = salvage_pct * inflation_factor * (total_depreciable_costs + total_nondepreciable_costs)
    inflated_other_rawmaterial = 0
    inflated_othervar = 0
    H2_price_nominal = 4.74027691  # not included

    # --------------------------------------DCF Calculation OUTPUT----------------------------------------------

    Initial_Data = {
        'Year': np.arange(assumed_startup_year - length_of_construction_period, assumed_startup_year + plant_life),
        'Analysis Year': np.arange(plant_life + 1)}
    df = pd.DataFrame(data=Initial_Data)

    df['Inflation Price Increase Factor'] = (1 + inflation_rate) ** (df['Year'] - assumed_startup_year)


    df['Revenue from Hydrogen Sales'] = 0
    df['Revenue from Hydrogen Sales'][df['Analysis Year'] == 0] = 0
    df['Revenue from Hydrogen Sales'][df['Analysis Year'] == 1] = H2_price_nominal * df[
        'Inflation Price Increase Factor'] * plant_annual_output * percent_revenue_during_startup
    df['Revenue from Hydrogen Sales'][(df['Analysis Year'] == 1) & (startup_time < 1)] = df['Inflation Price Increase Factor'] * H2_price_nominal * plant_annual_output * startup_time * percent_revenue_during_startup \
                                + H2_price_nominal * plant_annual_output * (1 - startup_time)
    df['Revenue from Hydrogen Sales'][df['Analysis Year'] > 1] = H2_price_nominal * plant_annual_output * df[
        'Inflation Price Increase Factor']


    df['Debt Financed Initial Depreciable Capital'] = 0
    df['Debt Financed Initial Depreciable Capital'][df['Analysis Year'] == 0] = -df[
        'Inflation Price Increase Factor'] * debt_financing * total_depreciable_costs * inflation_factor
    df['Debt Financed Initial Depreciable Capital'][df['Analysis Year'] >= 1] = 0


    df['Initial Equity Depreciable Capital'] = 0
    df['Initial Equity Depreciable Capital'][df['Analysis Year'] < 1] = -df[
        'Inflation Price Increase Factor'] * total_depreciable_costs_inflated * percent_equity_financing * percent_capital_spent_year_1




    df['Yearly Replacement Cost'] = 0
    df['Yearly Replacement Cost'][
        df['Analysis Year'] >= 1] = -unplanned_replacement_cost_factor * total_depreciable_costs * df[
        'Inflation Price Increase Factor'] * ((1 + inflation_rate) ** (assumed_startup_year - reference_year))
    df['Yearly Replacement Cost'][
        (df['Analysis Year'] == 8) | (df['Analysis Year'] == 15) | (df['Analysis Year'] == 22) | (
                df['Analysis Year'] == 29) | (df['Analysis Year'] == 36)] = -df[
        'Inflation Price Increase Factor'] * ((unplanned_replacement_cost_factor * total_depreciable_costs +
                                               0.15 * H2A_total_direct_capital_cost) * (1 + inflation_rate) ** (
                                                      assumed_startup_year - reference_year))  # /(CPInfactor*CEPCIinflator)




    df['Other Non-Depreciable Capital Cost'] = 0
    df['Other Non-Depreciable Capital Cost'][df['Analysis Year'] == 0] = -total_nondepreciable_costs * df[
        'Inflation Price Increase Factor'] * inflation_factor




    df['Salvage Value'] = 0
    df['Salvage Value'][df['Analysis Year'] == analysis_period] = df['Inflation Price Increase Factor'] * salvage_value




    df['Decommissioning Costs'] = 0
    df['Decommissioning Costs'][df['Analysis Year'] == analysis_period] = -df['Inflation Price Increase Factor'] * decom




    df['Fixed Operating Cost'] = 0
    df['Fixed Operating Cost'][df['Analysis Year'] >= 1] = -inflated_fixed * df['Inflation Price Increase Factor']
    df['Fixed Operating Cost'][(df['Analysis Year'] == 1) & (startup_time < 1)] = -(
            inflated_fixed * percent_fixed_operating_cost_during_startup * startup_time + (
            inflated_fixed * (1 - startup_time)))
    df['Fixed Operating Cost'][df['Analysis Year'] == startup_time] = -(
            inflated_fixed * percent_fixed_operating_cost_during_startup * df['Inflation Price Increase Factor'])




    df['Feedstock Cost'] = 0
    df['Feedstock Cost'][(df['Analysis Year'] == 1) & (startup_time < 1)] = -(
            total_energy_feedstock_unitcost * plant_annual_output * df[
        'Inflation Price Increase Factor'] * startup_time * percent_variable_operating_cost_during_startup + total_energy_feedstock_unitcost * plant_annual_output *
            df['Inflation Price Increase Factor'] * (1 - startup_time))
    df['Feedstock Cost'][df['Analysis Year'] <= startup_time] = -total_energy_feedstock_unitcost * plant_annual_output * \
                                                                df[
                                                                    'Inflation Price Increase Factor'] * percent_variable_operating_cost_during_startup
    df['Feedstock Cost'][df['Analysis Year'] >= 1] = -total_energy_feedstock_unitcost * plant_annual_output * df[
        'Inflation Price Increase Factor']




    df['Other Raw Material Cost'] = 0
    df['Other Raw Material Cost'][(df['Analysis Year'] == 1) & (startup_time < 1)] = -(inflated_other_rawmaterial * df[
        'Inflation Price Increase Factor'] * startup_time * percent_variable_operating_cost_during_startup + inflated_other_rawmaterial *
                                                                                       df[
                                                                                           'Inflation Price Increase Factor'] * (
                                                                                               1 - startup_time))
    df['Other Raw Material Cost'][df['Analysis Year'] <= startup_time] = -inflated_other_rawmaterial * df[
        'Inflation Price Increase Factor'] * percent_variable_operating_cost_during_startup
    df['Other Raw Material Cost'][df['Analysis Year'] >= 1] = -inflated_other_rawmaterial * df[
        'Inflation Price Increase Factor']




    df['Revenue from Byproduct Sales'] = 0
    df['Revenue from Byproduct Sales'][(df['Analysis Year'] == 1) & (startup_time < 1)] = (
            total_byproduct_unitprice * plant_annual_output * df[
        'Inflation Price Increase Factor'] * startup_time * percent_variable_operating_cost_during_startup + total_energy_feedstock_unitcost * plant_annual_output *
            df['Inflation Price Increase Factor'] * (1 - startup_time))
    df['Revenue from Byproduct Sales'][
        df['Analysis Year'] <= startup_time] = total_byproduct_unitprice * plant_annual_output * df[
        'Inflation Price Increase Factor'] * percent_variable_operating_cost_during_startup
    df['Revenue from Byproduct Sales'][df['Analysis Year'] >= 1] = total_byproduct_unitprice * plant_annual_output * df[
        'Inflation Price Increase Factor']




    Process_Water_Data = {'Year': np.arange(2014, 2061), 'Price ($(2016)/gal)': [0.002374951] * 47}
    Process_Water_Price = pd.DataFrame(data=Process_Water_Data)
    Process_Water_Price['Unit Cost'] = inflation_factor * water_usage_per_kgH2 * Process_Water_Price[
        'Price ($(2016)/gal)']
    Process_Water_Price = Process_Water_Price[(Process_Water_Price.Year >= assumed_startup_year - 1) & (
            Process_Water_Price.Year < assumed_startup_year + analysis_period)]
    Process_Water_Price.reset_index(drop=True, inplace=True)

    Industrial_Electricity_Data = {'Year': np.arange(2014, 2061),
                                   'Price ($(2012)/GJ LHV)': [20.19068, 19.43906, 18.77777252, 19.5374937444079,
                                                              19.0946888127265, 19.3408804840691, 19.7007267860663,
                                                              19.8092319270257, 20.4486283192551, 20.5134381492546,
                                                              20.4593613988262, \
                                                              20.9506225261977, 21.1633846923765, 21.2853222956886,
                                                              21.4236239592969, 21.5322826466083, 21.6077715306107,
                                                              21.6351454330386, 21.5857850199421, 21.508628925858,
                                                              21.4461184998711, \
                                                              21.5219523892571, 21.6002041597792, 21.5838723252605,
                                                              21.5886663835853, 21.6053745014483, 21.5547155790783,
                                                              21.4913246311096, 21.5162512700748, 21.5819141353634,
                                                              21.6632320938415, 21.7423672297963, \
                                                              21.8228245704493, 21.8950245295036, 21.97623917593,
                                                              22.0466032134787, 22.1148194977328, 22.1348766528058,
                                                              22.1549519988241, 22.1750455522859, 22.1951573297046,
                                                              22.2152873476085, 22.2374072845512, \
                                                              22.2595492464896, 22.2817132553544, 22.3038993330976,
                                                              22.3261075016936]}
    Industrial_Electricity_Price = pd.DataFrame(data=Industrial_Electricity_Data)
    if h2a_for_hopp:
        Industrial_Electricity_Price['Unit Cost'] = 0
    else:
        Industrial_Electricity_Price['Unit Cost'] = inflation_factor * electricity_usage * electricity_LHV * \
                                                    Industrial_Electricity_Price['Price ($(2012)/GJ LHV)']
    Industrial_Electricity_Price = Industrial_Electricity_Price[
        (Industrial_Electricity_Price.Year >= assumed_startup_year - 1) & (
                Industrial_Electricity_Price.Year < assumed_startup_year + analysis_period)]
    Industrial_Electricity_Price.reset_index(drop=True, inplace=True)

    df['Other Variable Operating Costs'] = 0
    df['Other Variable Operating Costs'][(df['Analysis Year'] == 1) & (startup_time < 1)] = -(((
                                                                                                       Industrial_Electricity_Price[
                                                                                                           'Unit Cost'] +
                                                                                                       Process_Water_Price[
                                                                                                           'Unit Cost']) * plant_annual_output + inflated_othervar) *
                                                                                              df[
                                                                                                  'Inflation Price Increase Factor'] * startup_time * percent_variable_operating_cost_during_startup + \
                                                                                              ((
                                                                                                       Industrial_Electricity_Price[
                                                                                                           'Unit Cost'] +
                                                                                                       Process_Water_Price[
                                                                                                           'Unit Cost']) * plant_annual_output + inflated_othervar) *
                                                                                              df[
                                                                                                  'Inflation Price Increase Factor'] * (
                                                                                                      1 - startup_time))
    df['Other Variable Operating Costs'][df['Analysis Year'] <= startup_time] = -((Industrial_Electricity_Price[
                                                                                       'Unit Cost'] +
                                                                                   Process_Water_Price[
                                                                                       'Unit Cost']) * plant_annual_output + inflated_othervar) * \
                                                                                df[
                                                                                    'Inflation Price Increase Factor'] * percent_variable_operating_cost_during_startup
    df['Other Variable Operating Costs'][df['Analysis Year'] > 1] = -df['Inflation Price Increase Factor'] * ((
                                                                                                                      Industrial_Electricity_Price[
                                                                                                                          'Unit Cost'] +
                                                                                                                      Process_Water_Price[
                                                                                                                          'Unit Cost']) * plant_annual_output + inflated_othervar)
    df['Other Variable Operating Costs'][df['Analysis Year'] == 0] = 0





    df['Working Capital Reserve'] = 0
    i = 1
    for i in range(i, len(df) - 1):
        df.loc[i, 'Working Capital Reserve'] = working_capital * ((df.loc[i, 'Fixed Operating Cost'] + df.loc[
            i, 'Feedstock Cost'] + df.loc[i, 'Other Raw Material Cost'] + df.loc[i, 'Revenue from Byproduct Sales'] +
                                                                   df.loc[i, 'Other Variable Operating Costs']) \
                                                                  - (df.loc[i - 1, 'Fixed Operating Cost'] + df.loc[
                    i - 1, 'Feedstock Cost'] + df.loc[i - 1, 'Other Raw Material Cost'] + df.loc[
                                                                         i - 1, 'Revenue from Byproduct Sales'] +
                                                                     df.loc[i - 1, 'Other Variable Operating Costs']))
    df['Working Capital Reserve'][df['Analysis Year'] == analysis_period] = -df.loc[0:analysis_period,
                                                                             'Working Capital Reserve'].sum()




    df['Debt Interest'] = 0
    df['Debt Interest'] = df.loc[0, 'Debt Financed Initial Depreciable Capital'] * interest_rate_on_debt




    df['Pre-Depreciation Income'] = 0
    df['Pre-Depreciation Income'] = df['Revenue from Hydrogen Sales'] + df['Salvage Value'] + df[
        'Decommissioning Costs'] + df['Fixed Operating Cost'] \
                                    + df['Feedstock Cost'] + df['Other Raw Material Cost'] + df[
                                        'Revenue from Byproduct Sales'] + df['Other Variable Operating Costs'] + df[
                                        'Debt Interest']




    # THREE_YEAR = [0.3333, 0.4445, 0.1481, 0.0741]
    # FIVE_YEAR = [0.2, 0.32, 0.1920, 0.1152, 0.1152, 0.0576]
    # SEVEN_YEAR = [0.1429,0.2449, 0.1749, 0.1249, 0.0893, 0.0892, 0.0893, 0.0446]
    # TEN_YEAR = [0.1, 0.18, 0.144, 0.1152, 0.0922, 0.0737, 0.0655, 0.0655, 0.0656, 0.0655, 0.0328]
    # FIFTHEEN_YEAR = [0.05, 0.095, 0.0855, 0.077, 0.0693, 0.0623, 0.059, 0.059, 0.0591, 0.059, 0.0591, 0.059, 0.0591, 0.059, 0.0591, 0.0295]
    TWENTY_YEAR = [0.0375, 0.07219, 0.06677, 0.06177, 0.05713, 0.05285, 0.04888, 0.04522, 0.04462, 0.04461, 0.04462,
                   0.04461, 0.04462, 0.04461, 0.04462, 0.04461, 0.04462, 0.04461, 0.04462, 0.04461, 0.02231]

    Depr_Initial_Data = {'Analysis Year': np.arange(plant_life + 21)}
    Depreciation_Table = pd.DataFrame(data=Depr_Initial_Data,
                                      columns=['Annual Depreciable Capital', 'Analysis Year', '1', '2', '3', '4', '5',
                                               '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18',
                                               '19', '20', '21'])
    Depreciation_Table['Annual Depreciable Capital'] = 0
    Depreciation_Table['Annual Depreciable Capital'][Depreciation_Table['Analysis Year'] == 1] = -(
            df.loc[0, 'Debt Financed Initial Depreciable Capital'] + df[
        'Initial Equity Depreciable Capital'].sum()) - df['Yearly Replacement Cost']
    Depreciation_Table['Annual Depreciable Capital'][
        (Depreciation_Table['Analysis Year'] > 1) & (Depreciation_Table['Analysis Year'] < plant_life + 1)] = -df[
        'Yearly Replacement Cost']
    Depreciation_Table['Annual Depreciable Capital'][Depreciation_Table['Analysis Year'] > plant_life] = 0

    Depreciation_Table['1'] = 0
    Depreciation_Table['1'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                           'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                           0]
    Depreciation_Table['2'] = 0
    Depreciation_Table['2'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                           'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                           1]
    Depreciation_Table['3'] = 0
    Depreciation_Table['3'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                           'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                           2]
    Depreciation_Table['4'] = 0
    Depreciation_Table['4'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                           'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                           3]
    Depreciation_Table['5'] = 0
    Depreciation_Table['5'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                           'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                           4]
    Depreciation_Table['6'] = 0
    Depreciation_Table['6'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                           'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                           5]
    Depreciation_Table['7'] = 0
    Depreciation_Table['7'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                           'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                           6]
    Depreciation_Table['8'] = 0
    Depreciation_Table['8'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                           'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                           7]
    Depreciation_Table['9'] = 0
    Depreciation_Table['9'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                           'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                           8]
    Depreciation_Table['10'] = 0
    Depreciation_Table['10'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                            'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                            9]
    Depreciation_Table['11'] = 0
    Depreciation_Table['11'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                            'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                            10]
    Depreciation_Table['12'] = 0
    Depreciation_Table['12'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                            'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                            11]
    Depreciation_Table['13'] = 0
    Depreciation_Table['13'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                            'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                            12]
    Depreciation_Table['14'] = 0
    Depreciation_Table['14'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                            'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                            13]
    Depreciation_Table['15'] = 0
    Depreciation_Table['15'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                            'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                            14]
    Depreciation_Table['16'] = 0
    Depreciation_Table['16'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                            'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                            15]
    Depreciation_Table['17'] = 0
    Depreciation_Table['17'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                            'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                            16]
    Depreciation_Table['18'] = 0
    Depreciation_Table['18'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                            'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                            17]
    Depreciation_Table['19'] = 0
    Depreciation_Table['19'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                            'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                            18]
    Depreciation_Table['20'] = 0
    Depreciation_Table['20'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                            'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                            19]
    Depreciation_Table['21'] = 0
    Depreciation_Table['21'][Depreciation_Table['Analysis Year'] > 0] = Depreciation_Table[
                                                                            'Annual Depreciable Capital'] * TWENTY_YEAR[
                                                                            20]

    Depreciation_Table['Depreciation Charge'] = 0
    for i in range(1, useful_life+21):
        Depreciation_Table.loc[i, 'Depreciation Charge'] = Depreciation_Table.loc[i, '1'] + Depreciation_Table.loc[
            i - 1, '2'] + Depreciation_Table.loc[(i - 2) if (i - 2) > 0 else 0, '3'] + Depreciation_Table.loc[
                                                               (i - 3) if (i - 3) > 0 else 0, '4'] + \
                                                           Depreciation_Table.loc[(i - 4) if (i - 4) > 0 else 0, '5'] + \
                                                           Depreciation_Table.loc[(i - 5) if (i - 5) > 0 else 0, '6'] + \
                                                           Depreciation_Table.loc[(i - 6) if (i - 6) > 0 else 0, '7'] + \
                                                           Depreciation_Table.loc[(i - 7) if (i - 7) > 0 else 0, '8'] + \
                                                           Depreciation_Table.loc[(i - 8) if (i - 8) > 0 else 0, '9'] + \
                                                           Depreciation_Table.loc[(i - 9) if (i - 9) > 0 else 0, '10'] + \
                                                           Depreciation_Table.loc[
                                                               (i - 10) if (i - 10) > 0 else 0, '11'] + \
                                                           Depreciation_Table.loc[
                                                               (i - 11) if (i - 11) > 0 else 0, '12'] + \
                                                           Depreciation_Table.loc[
                                                               (i - 12) if (i - 12) > 0 else 0, '13'] + \
                                                           Depreciation_Table.loc[
                                                               (i - 13) if (i - 13) > 0 else 0, '14'] + \
                                                           Depreciation_Table.loc[
                                                               (i - 14) if (i - 14) > 0 else 0, '15'] + \
                                                           Depreciation_Table.loc[
                                                               (i - 15) if (i - 15) > 0 else 0, '16'] + \
                                                           Depreciation_Table.loc[
                                                               (i - 16) if (i - 16) > 0 else 0, '17'] + \
                                                           Depreciation_Table.loc[
                                                               (i - 17) if (i - 17) > 0 else 0, '18'] + \
                                                           Depreciation_Table.loc[
                                                               (i - 18) if (i - 18) > 0 else 0, '19'] + \
                                                           Depreciation_Table.loc[
                                                               (i - 19) if (i - 19) > 0 else 0, '20'] + \
                                                           Depreciation_Table.loc[(i - 20) if (i - 20) > 0 else 0, '21']




    df['Depreciation Charge'] = 0
    df['Depreciation Charge'][df['Analysis Year'] < plant_life] = -Depreciation_Table['Depreciation Charge']
    df['Depreciation Charge'][df['Analysis Year'] == plant_life] = -Depreciation_Table.loc[
        useful_life, 'Depreciation Charge'] - Depreciation_Table.loc[useful_life+1:useful_life+21, 'Depreciation Charge'].sum()




    df['Taxable Income'] = 0
    df['Taxable Income'] = df['Pre-Depreciation Income'] + df['Depreciation Charge']




    df['Tax Rate (%)'] = total_tax_rate





    df['Total Taxes'] = 0
    df['Total Taxes'] = -df['Taxable Income'] * df['Tax Rate (%)']

    df['After Income Tax'] = 0
    df['After Income Tax'] = df['Pre-Depreciation Income'] + df['Total Taxes']


    df['Principal Payment'] = 0
    df['Principal Payment'][df['Analysis Year'] >= 1] = 0
    df['Principal Payment'][df['Analysis Year'] == analysis_period] = df.loc[
        0, 'Debt Financed Initial Depreciable Capital']



    df['After-Tax Post-Depreciation Cash Flow'] = 0
    df['After-Tax Post-Depreciation Cash Flow'] = df['Initial Equity Depreciable Capital'] + df[
        'Yearly Replacement Cost'] + df['Working Capital Reserve'] \
                                                  + df['Other Non-Depreciable Capital Cost'] + df[
                                                      'Pre-Depreciation Income'] + df['Total Taxes'] + df[
                                                      'Principal Payment']



    df['Cumulative Cash Flow'] = 0
    df['Cumulative Cash Flow'][df['Analysis Year'] == 0] = df.loc[0, 'After-Tax Post-Depreciation Cash Flow']
    i = 1
    for i in range(i, len(df)):
        df.loc[i, 'Cumulative Cash Flow'] = df.loc[i, 'After-Tax Post-Depreciation Cash Flow'] + df.loc[
            i - 1, 'Cumulative Cash Flow']

    df['Pre-Tax Cash Flow'] = 0
    df['Pre-Tax Cash Flow'] = df['Initial Equity Depreciable Capital'] + df['Yearly Replacement Cost'] + df[
        'Working Capital Reserve'] \
                              + df['Other Non-Depreciable Capital Cost'] + df['Pre-Depreciation Income'] + df[
                                  'Principal Payment']


    df['H2 Sales (kg/year)'] = 0
    df['H2 Sales (kg/year)'][(df['Analysis Year'] == 1) & (
            startup_time < 1)] = plant_annual_output * startup_time * percent_revenue_during_startup + plant_annual_output * (
            1 - startup_time)
    df['H2 Sales (kg/year)'][df['Analysis Year'] == startup_time] = plant_annual_output * percent_revenue_during_startup
    df['H2 Sales (kg/year)'][df['Analysis Year'] > 1] = plant_annual_output

    df.loc['TOTAL'] = df.sum(numeric_only=True, axis=0)


    def npv(rate, values):
        npv = (values / ((1 + rate) ** np.arange(1, len(values) + 1))).sum(axis=0)
        return npv


    df.loc['NPV of Cashflow', 'Revenue from Hydrogen Sales'] = npv(nominal_IRR, (df['Revenue from Hydrogen Sales'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Revenue from Hydrogen Sales'] = (df['Revenue from Hydrogen Sales'].tolist())[0] + npv(
        nominal_IRR, (df['Revenue from Hydrogen Sales'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Debt Financed Initial Depreciable Capital'] = npv(nominal_IRR, (df['Debt Financed Initial Depreciable Capital'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Debt Financed Initial Depreciable Capital'] = (df['Debt Financed Initial Depreciable Capital'].tolist())[0] + npv(nominal_IRR, (df[ \
        'Debt Financed Initial Depreciable Capital'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Initial Equity Depreciable Capital'] = npv(nominal_IRR, (df['Initial Equity Depreciable Capital'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Initial Equity Depreciable Capital'] = (df['Initial Equity Depreciable Capital'].tolist())[
                                                                            0] + npv(nominal_IRR, (df['Debt Financed Initial Depreciable Capital'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Yearly Replacement Cost'] = npv(nominal_IRR, (df['Yearly Replacement Cost'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Yearly Replacement Cost'] = (df['Yearly Replacement Cost'].tolist())[0] + npv(nominal_IRR,
                                                                                                               (df['Yearly Replacement Cost'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Working Capital Reserve'] = npv(nominal_IRR, (df['Working Capital Reserve'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Working Capital Reserve'] = (df['Working Capital Reserve'].tolist())[0] + npv(nominal_IRR,
                                                                                                               (df['Working Capital Reserve'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Other Non-Depreciable Capital Cost'] = npv(nominal_IRR, (df['Other Non-Depreciable Capital Cost'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Other Non-Depreciable Capital Cost'] = (df['Other Non-Depreciable Capital Cost'].tolist())[0] + \
                                                                        npv(nominal_IRR, (df['Other Non-Depreciable Capital Cost'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Salvage Value'] = npv(nominal_IRR, (df['Salvage Value'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Salvage Value'] = (df['Salvage Value'].tolist())[0] + npv(nominal_IRR,(df['Salvage Value'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Decommissioning Costs'] = npv(nominal_IRR, (df['Decommissioning Costs'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Decommissioning Costs'] = (df['Decommissioning Costs'].tolist())[0] + npv(nominal_IRR, (df['Decommissioning Costs'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Fixed Operating Cost'] = npv(nominal_IRR, (df['Fixed Operating Cost'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Fixed Operating Cost'] = (df['Fixed Operating Cost'].tolist())[0] + npv(nominal_IRR, (df['Fixed Operating Cost'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Feedstock Cost'] = npv(nominal_IRR, (df['Feedstock Cost'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Feedstock Cost'] = (df['Feedstock Cost'].tolist())[0] + npv(nominal_IRR, (df['Feedstock Cost'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Other Raw Material Cost'] = npv(nominal_IRR, (df['Other Raw Material Cost'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Other Raw Material Cost'] = (df['Other Raw Material Cost'].tolist())[0] + npv(nominal_IRR,(df['Other Raw Material Cost'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Revenue from Byproduct Sales'] = npv(nominal_IRR,(df['Revenue from Byproduct Sales'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Revenue from Byproduct Sales'] = (df['Revenue from Byproduct Sales'].tolist())[0] + npv(nominal_IRR, (df['Revenue from Byproduct Sales'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Other Variable Operating Costs'] = npv(nominal_IRR,(df['Other Variable Operating Costs'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Other Variable Operating Costs'] = (df['Other Variable Operating Costs'].tolist())[0] + npv(nominal_IRR, (df['Other Variable Operating Costs'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Debt Interest'] = npv(nominal_IRR, (df['Debt Interest'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Debt Interest'] = (df['Debt Interest'].tolist())[0] + npv(nominal_IRR, (df['Debt Interest'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Pre-Depreciation Income'] = npv(nominal_IRR, (df['Pre-Depreciation Income'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Pre-Depreciation Income'] = (df['Pre-Depreciation Income'].tolist())[0] + npv(nominal_IRR, (df['Pre-Depreciation Income'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Depreciation Charge'] = npv(nominal_IRR, (df['Depreciation Charge'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Depreciation Charge'] = (df['Depreciation Charge'].tolist())[0] + npv(nominal_IRR, (df['Depreciation Charge'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Taxable Income'] = npv(nominal_IRR, (df['Taxable Income'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Taxable Income'] = (df['Taxable Income'].tolist())[0] + npv(nominal_IRR, (df['Taxable Income'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Total Taxes'] = npv(nominal_IRR, (df['Total Taxes'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Total Taxes'] = (df['Total Taxes'].tolist())[0] + npv(nominal_IRR,
                                                                                       (df['Total Taxes'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'After Income Tax'] = npv(nominal_IRR, (df['After Income Tax'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'After Income Tax'] = (df['After Income Tax'].tolist())[0] + npv(nominal_IRR, (df['After Income Tax'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Principal Payment'] = npv(nominal_IRR, (df['Principal Payment'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Principal Payment'] = (df['Principal Payment'].tolist())[0] + npv(nominal_IRR, (df['Principal Payment'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'After-Tax Post-Depreciation Cash Flow'] = npv(nominal_IRR, (df['After-Tax Post-Depreciation Cash Flow'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'After-Tax Post-Depreciation Cash Flow'] = \
        (df['After-Tax Post-Depreciation Cash Flow'].tolist())[0] + npv(nominal_IRR,
                                                                        (df['After-Tax Post-Depreciation Cash Flow'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Cumulative Cash Flow'] = npv(nominal_IRR, (df['Cumulative Cash Flow'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Cumulative Cash Flow'] = (df['Cumulative Cash Flow'].tolist())[0] + npv(nominal_IRR, (df['Cumulative Cash Flow'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'Pre-Tax Cash Flow'] = npv(nominal_IRR, (df['Pre-Tax Cash Flow'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'Pre-Tax Cash Flow'] = (df['Pre-Tax Cash Flow'].tolist())[0] + npv(nominal_IRR, (df['Pre-Tax Cash Flow'].tolist())[1:useful_life+1])

    df.loc['NPV of Cashflow', 'H2 Sales (kg/year)'] = npv(nominal_IRR, (df['H2 Sales (kg/year)'].tolist())[:useful_life+1])
    df.loc['Discounted Values', 'H2 Sales (kg/year)'] = (df['H2 Sales (kg/year)'].tolist())[0] + npv(after_tax_real_IRR, (df['H2 Sales (kg/year)'].tolist())[1:useful_life+1])

    final_data = pd.DataFrame(
        columns=['Capital Costs', 'Depreciation', 'Principal', 'Operation Costs', 'Tax Incentives', 'H2 Sales (kg)'])
    final_data.loc['Discounted Values', 'Capital Costs'] = -(
            df.loc['Discounted Values', 'Initial Equity Depreciable Capital'] + df.loc[
        'Discounted Values', 'Yearly Replacement Cost'] + df.loc['Discounted Values', 'Working Capital Reserve'] +
            df.loc['Discounted Values', 'Other Non-Depreciable Capital Cost'])
    final_data.loc['Discounted Values', 'Depreciation'] = -df.loc['Discounted Values', 'Depreciation Charge']
    final_data.loc['Discounted Values', 'Principal'] = -df.loc['Discounted Values', 'Principal Payment']
    final_data.loc['Discounted Values', 'Operation Costs'] = -(
            df.loc['Discounted Values', 'Salvage Value'] + df.loc['Discounted Values', 'Decommissioning Costs'] +
            df.loc['Discounted Values', 'Fixed Operating Cost'] + df.loc['Discounted Values', 'Feedstock Cost'] + \
            df.loc['Discounted Values', 'Other Raw Material Cost'] + df.loc[
                'Discounted Values', 'Revenue from Byproduct Sales'] + df.loc[
                'Discounted Values', 'Other Variable Operating Costs'] + df.loc['Discounted Values', 'Debt Interest'])
    final_data.loc['Discounted Values', 'Tax Incentives'] = tax_credit
    final_data.loc['Discounted Values', 'H2 Sales (kg)'] = df.loc['Discounted Values', 'H2 Sales (kg/year)']

    final_data.loc['Tax Coefficient', 'Capital Costs'] = 1
    final_data.loc['LCOH Cost Contribution', 'Capital Costs'] = final_data.loc['Discounted Values', 'Capital Costs'] * \
                                                                final_data.loc['Tax Coefficient', 'Capital Costs']

    final_data.loc['Tax Coefficient', 'Depreciation'] = -total_tax_rate
    final_data.loc['LCOH Cost Contribution', 'Depreciation'] = final_data.loc['Discounted Values', 'Depreciation'] * \
                                                               final_data.loc['Tax Coefficient', 'Depreciation']

    final_data.loc['Tax Coefficient', 'Principal'] = 1
    final_data.loc['LCOH Cost Contribution', 'Principal'] = final_data.loc['Discounted Values', 'Principal'] * \
                                                            final_data.loc['Tax Coefficient', 'Principal']

    final_data.loc['Tax Coefficient', 'Operation Costs'] = 1 - total_tax_rate
    final_data.loc['LCOH Cost Contribution', 'Operation Costs'] = final_data.loc['Discounted Values', 'Operation Costs'] * \
                                                                  final_data.loc['Tax Coefficient', 'Operation Costs']

    final_data.loc['Tax Coefficient', 'Tax Incentives'] = -1
    final_data.loc['LCOH Cost Contribution', 'Tax Incentives'] = final_data.loc['Discounted Values', 'Tax Incentives'] * \
                                                                 final_data.loc['Tax Coefficient', 'Tax Incentives']

    final_data.loc['Tax Coefficient', 'H2 Sales (kg)'] = 1 - total_tax_rate
    final_data.loc['LCOH Cost Contribution', 'H2 Sales (kg)'] = final_data.loc['Discounted Values', 'H2 Sales (kg)'] * \
                                                                final_data.loc['Tax Coefficient', 'H2 Sales (kg)']

    Final_Hydrogen_Cost_Real = ((final_data.loc['LCOH Cost Contribution', 'Capital Costs'] + final_data.loc[
        'LCOH Cost Contribution', 'Depreciation'] + final_data.loc['LCOH Cost Contribution', 'Principal'] + \
                                 final_data.loc['LCOH Cost Contribution', 'Operation Costs'] + final_data.loc[
                                     'LCOH Cost Contribution', 'Tax Incentives']) / final_data.loc[
                                    'LCOH Cost Contribution', 'H2 Sales (kg)']) * (
                                       1 + inflation_rate) ** length_of_construction_period / inflation_factor
    # print(Final_Hydrogen_Cost_Real)

    Cost_Breakdown = pd.DataFrame(columns=['After Tax Present Value', '% of Total', '$/kg of H2'])
    Cost_Breakdown.loc['Capital Related Costs', 'After Tax Present Value'] = -(((df.loc[
                                                                                     'NPV of Cashflow', 'Initial Equity Depreciable Capital'] +
                                                                                 df.loc[
                                                                                     'NPV of Cashflow', 'Yearly Replacement Cost'] +
                                                                                 df.loc[
                                                                                     'NPV of Cashflow', 'Working Capital Reserve'] +
                                                                                 df.loc[
                                                                                     'NPV of Cashflow', 'Other Non-Depreciable Capital Cost']) + \
                                                                                df.loc[
                                                                                    'NPV of Cashflow', 'Salvage Value'] * (
                                                                                        1 - total_tax_rate) + df.loc[
                                                                                    'NPV of Cashflow', 'Debt Interest'] * (
                                                                                        1 - total_tax_rate) + df.loc[
                                                                                    'NPV of Cashflow', 'Depreciation Charge'] * (
                                                                                    -total_tax_rate) + df.loc[
                                                                                    'NPV of Cashflow', 'Principal Payment']) / (
                                                                                       1 - total_tax_rate))
    Cost_Breakdown.loc['Decommissioning Costs', 'After Tax Present Value'] = -df.loc[
        'NPV of Cashflow', 'Decommissioning Costs']
    Cost_Breakdown.loc['Fixed O&M', 'After Tax Present Value'] = -df.loc['NPV of Cashflow', 'Fixed Operating Cost']
    Cost_Breakdown.loc['Feedstock Costs', 'After Tax Present Value'] = df.loc['NPV of Cashflow', 'Feedstock Cost']
    Cost_Breakdown.loc['Other Raw Material Costs', 'After Tax Present Value'] = df.loc[
        'NPV of Cashflow', 'Other Raw Material Cost']
    Cost_Breakdown.loc['Byproduct Credits', 'After Tax Present Value'] = df.loc[
        'NPV of Cashflow', 'Revenue from Byproduct Sales']
    Cost_Breakdown.loc['Other Variable Costs (Utilities)', 'After Tax Present Value'] = -df.loc[
        'NPV of Cashflow', 'Other Variable Operating Costs']

    Cost_Breakdown['% of Total'] = (Cost_Breakdown['After Tax Present Value'] / Cost_Breakdown[
        'After Tax Present Value'].sum()) * 100
    Cost_Breakdown['$/kg of H2'] = (Cost_Breakdown['% of Total'] / 100) * Final_Hydrogen_Cost_Real
    Cost_Breakdown.loc['TOTAL'] = Cost_Breakdown.sum(axis=0)

    # Depreciation_Table.to_csv('Depreciation_Table.csv')
    # df.to_csv('CashFlow.csv')
    # final_data.to_csv('LCOH_Cost_Contribution')
    # Cost_Breakdown.to_csv('Cost_Breakdown.csv')

    # print(Cost_Breakdown)
    feedstock_cost_h2_levelized = Cost_Breakdown.loc['Other Variable Costs (Utilities)', '$/kg of H2']
    results = dict()
    results['Capital Related Costs'] = Cost_Breakdown.loc['Capital Related Costs', '$/kg of H2']
    results['Fixed O&M'] = Cost_Breakdown.loc['Fixed O&M', '$/kg of H2']
    results['Variable Costs/Feedstock'] =Cost_Breakdown.loc['Other Variable Costs (Utilities)', '$/kg of H2']
    results['Total Hydrogen Cost ($/kgH2)'] = Final_Hydrogen_Cost_Real
    results['Total H2A Cost'] = Cost_Breakdown.sum(axis=0)
    results['peak_daily_production_rate'] = peak_daily_production_rate
    results['electrolyzer_size'] = stack_input_power
    results['total_plant_size'] = total_system_input
    results['scaled_total_installed_cost'] = scaled_total_installed_cost
    results['scaled_total_installed_cost_kw'] = scaled_total_installed_cost/(total_system_input*1000)
    results['expenses_annual_cashflow'] = df['After-Tax Post-Depreciation Cash Flow'] - df['Pre-Depreciation Income']

    return results